SystemOrganization addCategory: #'Metacello-Core-Exceptions'!
SystemOrganization addCategory: #'Metacello-Core-Loaders'!
SystemOrganization addCategory: #'Metacello-Core-Members'!
SystemOrganization addCategory: #'Metacello-Core-Model'!
SystemOrganization addCategory: #'Metacello-Core-Specs'!

!Collection methodsFor: '*metacello-core' stamp: 'dkh 6/6/2009 11:45'!
addToMetacelloPackages: aMetacelloPackagesSpec

	self do: [:each | each addToMetacelloPackages: aMetacelloPackagesSpec ]! !

!Collection methodsFor: '*metacello-core' stamp: 'dkh 6/6/2009 11:46'!
mergeIntoMetacelloPackages: aMetacelloPackagesSpec

	self do: [:each | each mergeIntoMetacelloPackages: aMetacelloPackagesSpec ]! !

!Collection methodsFor: '*metacello-core' stamp: 'dkh 6/6/2009 11:42'!
removeFromMetacelloPackages: aMetacelloPackagesSpec

	self do: [:each | each removeFromMetacelloPackages: aMetacelloPackagesSpec ]! !

!Collection methodsFor: '*metacello-core' stamp: 'dkh 9/23/2009 08:46'!
setIncludesInMetacelloPackage: aMetacelloPackageSpec

	aMetacelloPackageSpec setIncludes: self asArray.! !

!Collection methodsFor: '*metacello-core' stamp: 'dkh 6/8/2009 19:02'!
setLoadsInMetacelloProject: aMetacelloPackageSpec

	aMetacelloPackageSpec setLoads: self asArray.! !

!Collection methodsFor: '*metacello-core' stamp: 'dkh 6/6/2009 10:08'!
setRequiresInMetacelloPackage: aMetacelloPackageSpec

	aMetacelloPackageSpec setRequires: self asArray.! !

Error subclass: #MetacelloProjectSpecLoadError
	instanceVariableNames: 'projectSpec'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Exceptions'!

MetacelloProjectSpecLoadError subclass: #MetacelloProjectSpecLoadConflict
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Exceptions'!

!MetacelloProjectSpecLoadConflict methodsFor: 'private' stamp: 'DaleHenrichs 2/5/2010 16:16'!
isResumable
	"Determine whether an exception is resumable."

	^ true! !

!MetacelloProjectSpecLoadError class methodsFor: 'instance creation' stamp: 'DaleHenrichs 1/19/2010 11:28'!
projectSpec: aMetacelloMCProjectSpec

	^self new
		projectSpec: aMetacelloMCProjectSpec;
		yourself! !

!MetacelloProjectSpecLoadError methodsFor: 'private' stamp: 'DaleHenrichs 2/5/2010 16:18'!
isResumable
	"Determine whether an exception is resumable."

	^ false! !

!MetacelloProjectSpecLoadError methodsFor: 'accessing' stamp: 'DaleHenrichs 1/19/2010 11:29'!
projectSpec

	^projectSpec! !

!MetacelloProjectSpecLoadError methodsFor: 'accessing' stamp: 'DaleHenrichs 1/19/2010 11:29'!
projectSpec: aMetacelloMCProjectSpec

	projectSpec := aMetacelloMCProjectSpec! !

!BlockClosure methodsFor: '*metacello-core' stamp: 'dkh 10/7/2009 00:12'!
setAuthorInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setAuthorWithBlock: self! !

!BlockClosure methodsFor: '*metacello-core' stamp: 'dkh 10/7/2009 00:12'!
setBlessingInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setBlessingWithBlock: self! !

!BlockClosure methodsFor: '*metacello-core' stamp: 'dkh 10/7/2009 00:14'!
setDescriptionInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setDescriptionWithBlock: self! !

!BlockClosure methodsFor: '*metacello-core' stamp: 'dkh 10/9/2009 11:28'!
setPackage: aString withInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setPackage: aString withBlock: self! !

!BlockClosure methodsFor: '*metacello-core' stamp: 'dkh 10/9/2009 11:17'!
setProject: aString withInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setProject: aString withBlock: self! !

!BlockClosure methodsFor: '*metacello-core' stamp: 'dkh 10/7/2009 00:15'!
setTimestampInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setTimestampWithBlock: self! !

Magnitude subclass: #MetacelloVersion
	instanceVariableNames: 'spec versionNumber importedVersions versionStatus'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Model'!

!MetacelloVersion class methodsFor: 'instance creation' stamp: 'tg 8/30/2009 18:07'!
fromSpec: aMetacelloVersionSpec

	^(self fromString: aMetacelloVersionSpec versionString) 
		spec: aMetacelloVersionSpec;
		yourself! !

!MetacelloVersion class methodsFor: 'instance creation' stamp: 'dkh 8/31/2009 05:13'!
fromString: aString

	^self new versionNumber: aString asMetacelloVersionNumber! !

!MetacelloVersion methodsFor: 'comparing' stamp: 'dkh 6/14/2009 09:40'!
< aMetacelloVersion

	aMetacelloVersion species = self species ifFalse: [ ^false ].
	^self versionNumber < aMetacelloVersion versionNumber! !

!MetacelloVersion methodsFor: 'comparing' stamp: 'dkh 6/14/2009 09:41'!
= aMetacelloVersion

	aMetacelloVersion species = self species ifFalse: [ ^false ].
	^self versionNumber = aMetacelloVersion versionNumber! !

!MetacelloVersion methodsFor: 'querying' stamp: 'dkh 10/13/2009 13:00'!
author

	^self spec author value! !

!MetacelloVersion methodsFor: 'querying' stamp: 'dkh 8/9/2009 11:48'!
blessing

	^self spec blessing value! !

!MetacelloVersion methodsFor: 'private' stamp: 'DaleHenrichs 2/8/2010 11:33'!
computeVersionStatus

	"all projects and packages are loaded and match specification"
	self isAllLoadedToSpec ifTrue: [ ^#allLoadedToSpec ].
	"all loaded projects and packages match specifications (at least one package loaded)"
	self isLoadedToSpec ifTrue: [ ^#loadedToSpec ].
	"all loaded projects and packages match constraints (at least one package loaded)"
	self isLoadedMatchConstraints ifTrue: [ ^#loadedMatchConstraints ].
	"at least one package loaded"
	self isSomethingLoaded ifTrue: [ ^#somethingLoaded ].
	^#noStatus! !

!MetacelloVersion methodsFor: 'querying' stamp: 'dkh 7/31/2009 11:48'!
description

	^self spec description value! !

!MetacelloVersion methodsFor: 'comparing' stamp: 'dkh 6/14/2009 09:41'!
hash

	^self versionNumber hash! !

!MetacelloVersion methodsFor: 'accessing' stamp: 'dkh 1/7/2010 18:48'!
importedVersions

	importedVersions == nil ifTrue: [ importedVersions := #() ].
	^importedVersions! !

!MetacelloVersion methodsFor: 'accessing' stamp: 'dkh 1/7/2010 18:57'!
importedVersions: aCollection

	importedVersions := aCollection! !

!MetacelloVersion methodsFor: 'testing' stamp: 'DaleHenrichs 2/5/2010 10:56'!
isAllLoadedToSpec
	"all projects and packages are loaded and match specification"
	
	^self spec isAllLoadedToSpec! !

!MetacelloVersion methodsFor: 'testing' stamp: 'DaleHenrichs 2/5/2010 11:08'!
isLoadedMatchConstraints
	"all loaded projects and packages match constraints"
	
	^self spec isLoadedMatchConstraints! !

!MetacelloVersion methodsFor: 'testing' stamp: 'DaleHenrichs 2/5/2010 11:07'!
isLoadedToSpec
	"all loaded projects and packages match specifications"
	
	^self spec isLoadedToSpec! !

!MetacelloVersion methodsFor: 'testing' stamp: 'DaleHenrichs 2/4/2010 10:19'!
isPossibleBaseline

	^self spec isPossibleBaseline! !

!MetacelloVersion methodsFor: 'testing' stamp: 'DaleHenrichs 2/5/2010 11:15'!
isSomethingLoaded
	"at least one project or package has been loaded"
	
	^self spec isSomethingLoaded! !

!MetacelloVersion methodsFor: 'actions' stamp: 'dkh 6/8/2009 17:45'!
load

	^self subclassResponsibility! !

!MetacelloVersion methodsFor: 'actions' stamp: 'dkh 6/8/2009 17:45'!
load: required

	^self subclassResponsibility! !

!MetacelloVersion methodsFor: 'testing' stamp: 'DaleHenrichs 2/8/2010 11:42'!
name

	^self versionString! !

!MetacelloVersion methodsFor: 'printing' stamp: 'DaleHenrichs 3/9/2010 16:32'!
printOn: aStream 

	| label vs |
	(vs := self versionStatus) == #somethingLoaded ifTrue: [ aStream nextPutAll: '<>' ].
	vs == #loadedMatchConstraints ifTrue: [ aStream nextPutAll: '>=' ].
	vs == #loadedToSpec ifTrue: [ aStream nextPut: $~ ].
	self versionNumber printOn: aStream.
	self spec ~~ nil
		ifTrue: [(label := self spec projectLabel) isEmpty
				ifFalse: [aStream nextPutAll: ' [' , label , ']']]! !

!MetacelloVersion methodsFor: 'querying' stamp: 'dkh 6/8/2009 16:03'!
project

	^self spec project! !

!MetacelloVersion methodsFor: 'accessing' stamp: 'DaleHenrichs 3/9/2010 16:32'!
projectLabel

	^self spec projectLabel! !

!MetacelloVersion methodsFor: 'accessing' stamp: 'dkh 05/04/2009 20:26'!
spec

	^spec! !

!MetacelloVersion methodsFor: 'accessing' stamp: 'dkh 05/04/2009 20:26'!
spec: aMetacellVersionSpec

	spec := aMetacellVersionSpec! !

!MetacelloVersion methodsFor: 'querying' stamp: 'dkh 10/13/2009 13:01'!
timestamp

	^self spec timestamp value! !

!MetacelloVersion methodsFor: 'actions' stamp: 'dkh 05/04/2009 20:26'!
unload

	^self spec unload! !

!MetacelloVersion methodsFor: 'querying' stamp: 'dkh 6/14/2009 09:39'!
versionNumber

	^versionNumber! !

!MetacelloVersion methodsFor: 'accessing' stamp: 'dkh 6/14/2009 09:39'!
versionNumber: aVersionNumber

	versionNumber := aVersionNumber! !

!MetacelloVersion methodsFor: 'accessing' stamp: 'dkh 6/15/2009 20:07'!
versionSpec

	^self spec! !

!MetacelloVersion methodsFor: 'accessing' stamp: 'DaleHenrichs 2/6/2010 16:49'!
versionStatus

	versionStatus == nil ifTrue: [ versionStatus := self computeVersionStatus].
	^versionStatus! !

!MetacelloVersion methodsFor: 'accessing' stamp: 'DaleHenrichs 2/4/2010 10:10'!
versionStatus: aSymbol

	versionStatus := aSymbol! !

!MetacelloVersion methodsFor: 'querying' stamp: 'dkh 8/31/2009 05:11'!
versionString

	^self versionNumber versionString! !

!MetacelloVersion methodsFor: 'comparing' stamp: 'dkh 6/14/2009 12:42'!
~> aMetacelloVersion

	aMetacelloVersion species = self species ifFalse: [ ^false ].
	^self versionNumber ~> aMetacelloVersion versionNumber! !

Magnitude variableSubclass: #MetacelloVersionNumber
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Model'!

!MetacelloVersionNumber class methodsFor: 'private' stamp: 'TestRunner 10/19/2009 10:22'!
extractNumericComponent: subString
	"$. separated components are integers"

	| number |
	number := [subString asNumber] 
						on: Error 
						do: [:ex | ex return: subString ].
	^number asString = subString
		ifTrue: [ number ]
		ifFalse: [ subString ]! !

!MetacelloVersionNumber class methodsFor: 'instance creation' stamp: 'dkh 8/13/2009 12:20'!
fromString: aString

	| new components |
	components := OrderedCollection new.
	(aString findTokens: '.') do: [:subString | | strs |
		strs := subString findTokens: '-'.
		"first subString token could be an integer"
		components add: (self extractNumericComponent: strs first).
		strs size > 1
			ifTrue: [
				"remaining are uncoditionally Strings, because of leading $-"
				components addAll: strs allButFirst ]].
	new := self new: components size.
	1 to: components size do: [:i | new at: i put: (components at: i) ].
	^new! !

!MetacelloVersionNumber methodsFor: 'comparing' stamp: 'dkh 11/3/2009 10:34'!
< aMetacelloVersionNumber

	| mySize aSize commonSize count more |
	aMetacelloVersionNumber species = self species ifFalse: [ ^false ].
	mySize := self size.
	aSize := aMetacelloVersionNumber size.
	commonSize :=  mySize min: aSize.
	count := 0.
	more := true.
	[ more and: [ count < commonSize ]] whileTrue: [
		(self at: count + 1) = (aMetacelloVersionNumber at: count + 1)
			ifTrue: [ count := count + 1 ]
			ifFalse: [ more := false ]].
	count < commonSize
		ifTrue: [ 
			^(self at: count + 1) 
				metacelloVersionComponentLessThan: (aMetacelloVersionNumber at: count + 1) ].
	mySize < aSize
		ifTrue: [ 
			mySize = 0 ifTrue: [ ^true ].
			"if the versions at commonSize are equal and the next version slot in aMetacelloVersionNumber 
			 is a string, then it's considered that I'm > aMetacelloVersionNumber
			 (i.e., '2.9.9' is greater than '2.9.9-alpha.2')"
			(self at: commonSize) = (aMetacelloVersionNumber at: commonSize)
				ifFalse: [ ^true ]. 
			^(aMetacelloVersionNumber at: commonSize+1) isString not]
		ifFalse: [ 
			mySize = aSize ifTrue: [ ^false ].
			aSize <= 0 ifTrue: [ ^false ].
			"if the versions at commonSize are equal and the next version slot is a string, 
			 then it's considered that I'm < aMetacelloVersionNumber
			 (i.e., '2.9.9-alpha.2' is less than '2.9.9')"
			(self at: commonSize) = (aMetacelloVersionNumber at: commonSize)
				ifFalse: [ ^false ].
			 ^(self at: commonSize+1) isString]! !

!MetacelloVersionNumber methodsFor: 'comparing' stamp: 'dkh 6/14/2009 09:21'!
= aMetacelloVersionNumber

	| mySize |
	aMetacelloVersionNumber species = self species ifFalse: [ ^false ].
	mySize := self size.
	mySize = aMetacelloVersionNumber size ifFalse: [ ^false ].
	1 to: mySize do: [:i |
		(self at: i) = (aMetacelloVersionNumber at: i) ifFalse: [ ^false ]].
	^true! !

!MetacelloVersionNumber methodsFor: 'accessing' stamp: 'dkh 6/14/2009 13:29'!
approximateBase

	| base |
	base := self copyFrom: 1 to: self size - 1.
	base at: base size put: (base at: base size) + 1.
	^base! !

!MetacelloVersionNumber methodsFor: 'converting' stamp: 'dkh 6/14/2009 09:29'!
asMetacelloVersionNumber

	^self! !

!MetacelloVersionNumber methodsFor: 'copying' stamp: 'dkh 6/14/2009 12:53'!
copyFrom: start to: stop 
	"Answer a copy of a subset of the receiver, starting from element at 
	index start until element at index stop."

	| newSize new j |
	newSize := stop - start + 1.
	new := self species new: newSize.
	j := 0.
	start to: stop do: [:i |
		new at: j + 1 put: (self at: i).
		j := j + 1 ].
	^new! !

!MetacelloVersionNumber methodsFor: 'enumerating' stamp: 'dkh 6/14/2009 09:37'!
do: aBlock 
	"Refer to the comment in Collection|do:."
	1 to: self size do:
		[:index | aBlock value: (self at: index)]! !

!MetacelloVersionNumber methodsFor: 'enumerating' stamp: 'dkh 6/14/2009 09:37'!
do: elementBlock separatedBy: separatorBlock
	"Evaluate the elementBlock for all elements in the receiver,
	and evaluate the separatorBlock between."

	| beforeFirst | 
	beforeFirst := true.
	self do:
		[:each |
		beforeFirst
			ifTrue: [beforeFirst := false]
			ifFalse: [separatorBlock value].
		elementBlock value: each]! !

!MetacelloVersionNumber methodsFor: 'comparing' stamp: 'dkh 06/15/2009 14:30'!
hash

"Returns a numeric hash key for the receiver."

| mySize interval hashValue |

(mySize := self size) == 0
  ifTrue: [ ^15243 ].

"Choose an interval so that we sample at most 5 elements of the receiver"
interval := ((mySize - 1) // 4) max: 1.

hashValue := 4459.
1 to: mySize by: interval do: [ :i | | anElement |
  anElement := self at: i.
  (anElement isKindOf: SequenceableCollection)
    ifTrue: [
      hashValue := (hashValue bitShift: -1) bitXor: anElement size.
      ]
    ifFalse: [
      hashValue := (hashValue bitShift: -1) bitXor: anElement hash.
      ].
  ].

^ hashValue abs! !

!MetacelloVersionNumber methodsFor: 'operations' stamp: 'dkh 10/7/2009 14:28'!
incrementMinorVersionNumber

	| int |
	self size to: 1 by: -1 do: [:index | 
		(int := self at: index) isString 
			ifFalse: [ 
				self at: index put: int + 1.
				^self ]].! !

!MetacelloVersionNumber methodsFor: 'comparing' stamp: 'dkh 12/21/2009 13:32'!
match: aVersionPattern
	"Answer whether the version number of the receiver matches the given pattern string.

	 A Metacello version number is made up of version sequences delimited by the characters $. and $-.
	 The $. introduces a numeric version sequence and $- introduces an alphanumeric version sequence.
	 
	 A version pattern is made up of version pattern match sequences. also delimited by the characters $. 
	 and $-.. Each pattern match sequence is tested against the corresponding version sequence of the 
	 receiver, using the 'standard' pattern matching rules. All sequences must answer true for a match.
	
	 The special pattern sequence '?' is a match for the corresponding version sequence and all subsequent 
	 version sequences. '?' as the version pattern matches all versions. No more version pattern 
	 sequences are permitted once the '?' sequence is used. If used, it is the last version pattern
	 sequence. "
	
	| patternVersion mySize patternSize |
	patternVersion := aVersionPattern asMetacelloVersionNumber.
	mySize := self size.
	patternSize := patternVersion size.
	mySize = patternSize 
		ifFalse: [ 
			mySize < patternSize ifTrue: [ ^false ].
			(patternVersion at: patternSize) ~= '?' ifTrue: [ ^false ].
			mySize := patternSize ].
	1 to: mySize do: [:i | | pattern |
		pattern := (patternVersion at: i) asString.
		pattern = '?'
			ifTrue: [i = mySize ifFalse: [ ^self error: 'Invalid version match pattern: ', aVersionPattern printString ]]
			ifFalse: [ (pattern match: (self at: i) asString)  ifFalse: [ ^false ]]].
	^true
"
  '1.1.1' asMetacelloVersionNumber match: '*.*.*'. -> true
  '1.1.1' asMetacelloVersionNumber match: '*.#.*'. -> true
  '1.10.1' asMetacelloVersionNumber match: '*.#.*'. -> false
  '1.1.1' asMetacelloVersionNumber match: '*.*'. -> false
  '1.1.1' asMetacelloVersionNumber match: '*.?'. -> true
  '1.0' asMetacelloVersionNumber match: '1.?'. -> true
  '2.0' asMetacelloVersionNumber match: '1.?'. -> false
  '1.1.1' asMetacelloVersionNumber match: '?'. -> true
  '1' asMetacelloVersionNumber match: '*.?'. -> false
  '1-alpha5.0' asMetacelloVersionNumber match: '1-alpha*.?'. -> true
  '1-alpha15.0.1' asMetacelloVersionNumber match: '1-alpha*.?'. -> true
  '1.1' asMetacelloVersionNumber match: '?.?'. -> ERROR: invalid version match pattern
"! !

!MetacelloVersionNumber methodsFor: 'printing' stamp: 'dkh 6/14/2009 09:59'!
printOn: aStream

	| beforeFirst | 
	beforeFirst := true.
	self do:
		[:each |
		beforeFirst
			ifTrue: [beforeFirst := false]
			ifFalse: [
				each isString
					ifTrue: [ aStream nextPut: $- ]
					ifFalse: [ aStream nextPut: $. ] ].
		aStream nextPutAll: each asString ]! !

!MetacelloVersionNumber methodsFor: 'accessing' stamp: 'dkh 6/14/2009 09:57'!
versionString

	| strm |
	strm := WriteStream on: String new.
	self printOn: strm.
	^strm contents! !

!MetacelloVersionNumber methodsFor: 'comparing' stamp: 'dkh 6/14/2009 13:30'!
~> aMetacelloVersionNumber

	aMetacelloVersionNumber size == 1 ifTrue: [ ^false ].
	^self >= aMetacelloVersionNumber and: [ self < aMetacelloVersionNumber approximateBase ]! !

!UndefinedObject methodsFor: '*metacello-core' stamp: 'dkh 11/3/2009 10:28'!
asMetacelloVersionNumber

	^MetacelloVersionNumber fromString: ''! !

!UndefinedObject methodsFor: '*metacello-core' stamp: 'dkh 9/13/2009 07:46'!
setAuthorInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setAuthor: self! !

!UndefinedObject methodsFor: '*metacello-core' stamp: 'dkh 6/5/2009 10:56'!
setBlessingInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setBlessing: self! !

!UndefinedObject methodsFor: '*metacello-core' stamp: 'dkh 6/5/2009 11:52'!
setDescriptionInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setDescription: self! !

!UndefinedObject methodsFor: '*metacello-core' stamp: 'dkh 12/11/2009 13:29'!
setProject: aString withInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setProject: aString withString: self! !

!UndefinedObject methodsFor: '*metacello-core' stamp: 'dkh 9/13/2009 07:47'!
setTimestampInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setTimestamp: self! !

!String methodsFor: '*metacello-core' stamp: 'dkh 06/03/2009 12:41'!
addToMetacelloPackages: aMetacelloPackagesSpec

	| spec |
	spec := 
		(aMetacelloPackagesSpec project packageSpec)
			file: self;
			yourself.
	aMetacelloPackagesSpec addMember: 
		(aMetacelloPackagesSpec addMember 
			name: spec name;
			spec: spec;
			yourself)! !

!String methodsFor: '*metacello-core' stamp: 'dkh 6/14/2009 09:31'!
asMetacelloVersionNumber

	^MetacelloVersionNumber fromString: self! !

!String methodsFor: '*metacello-core' stamp: 'dkh 6/6/2009 11:59'!
mergeIntoMetacelloPackages: aMetacelloPackagesSpec

	| spec |
	spec := 
		(aMetacelloPackagesSpec project packageSpec)
			file: self;
			yourself.
	aMetacelloPackagesSpec addMember: 
		(aMetacelloPackagesSpec mergeMember 
			name: spec name;
			spec: spec;
			yourself)! !

!String methodsFor: '*metacello-core-version comparison' stamp: 'dkh 8/13/2009 12:04'!
metacelloIntegerLessThanSelf: anInteger
	"integer version components are always '>' string component"

	^false! !

!String methodsFor: '*metacello-core-version comparison' stamp: 'dkh 8/13/2009 11:23'!
metacelloStringLessThanSelf: aString

	^aString < self! !

!String methodsFor: '*metacello-core-version comparison' stamp: 'dkh 8/13/2009 11:22'!
metacelloVersionComponentLessThan: aMetacelloVersonComponent

	^aMetacelloVersonComponent metacelloStringLessThanSelf: self! !

!String methodsFor: '*metacello-core' stamp: 'dkh 6/6/2009 11:59'!
removeFromMetacelloPackages: aMetacelloPackagesSpec

	aMetacelloPackagesSpec addMember: 
		(aMetacelloPackagesSpec removeMember 
			name: self;
			yourself)! !

!String methodsFor: '*metacello-core' stamp: 'dkh 10/7/2009 00:13'!
setAuthorInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setAuthorWithString: self! !

!String methodsFor: '*metacello-core' stamp: 'dkh 9/13/2009 07:45'!
setAuthorInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setAuthor: 
		(aMetacelloVersionSpec project valueHolderSpec
			value: self;
			yourself)! !

!String methodsFor: '*metacello-core' stamp: 'dkh 10/7/2009 00:10'!
setBlessingInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setBlessingWithString: self! !

!String methodsFor: '*metacello-core' stamp: 'dkh 6/5/2009 11:48'!
setBlessingInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setBlessing: 
		(aMetacelloVersionSpec project valueHolderSpec
			value: self;
			yourself)! !

!String methodsFor: '*metacello-core' stamp: 'dkh 10/7/2009 00:14'!
setDescriptionInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setDescriptionWithString: self! !

!String methodsFor: '*metacello-core' stamp: 'dkh 6/5/2009 11:51'!
setDescriptionInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setDescription: 
		(aMetacelloVersionSpec project valueHolderSpec
			value: self;
			yourself)! !

!String methodsFor: '*metacello-core' stamp: 'dkh 9/23/2009 08:46'!
setIncludesInMetacelloPackage: aMetacelloPackageSpec

	aMetacelloPackageSpec setIncludes: { self }.! !

!String methodsFor: '*metacello-core' stamp: 'dkh 6/8/2009 19:02'!
setLoadsInMetacelloProject: aMetacelloPackageSpec

	aMetacelloPackageSpec setLoads: { self }.! !

!String methodsFor: '*metacello-core' stamp: 'dkh 10/9/2009 11:28'!
setPackage: aString withInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setPackage: aString withString: self! !

!String methodsFor: '*metacello-core' stamp: 'dkh 10/9/2009 11:16'!
setProject: aString withInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setProject: aString withString: self! !

!String methodsFor: '*metacello-core' stamp: 'dkh 6/6/2009 10:08'!
setRequiresInMetacelloPackage: aMetacelloPackageSpec

	aMetacelloPackageSpec setRequires: { self }.! !

!String methodsFor: '*metacello-core' stamp: 'dkh 10/7/2009 00:15'!
setTimestampInMetacelloConfig: aMetacelloConfig

	aMetacelloConfig setTimestampWithString: self! !

!String methodsFor: '*metacello-core' stamp: 'dkh 9/13/2009 07:47'!
setTimestampInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setTimestamp: 
		(aMetacelloVersionSpec project valueHolderSpec
			value: self;
			yourself)! !

!Integer methodsFor: '*metacello-core' stamp: 'dkh 8/13/2009 10:38'!
metacelloIntegerLessThanSelf: anInteger

	^anInteger < self! !

!Integer methodsFor: '*metacello-core' stamp: 'dkh 8/13/2009 12:04'!
metacelloStringLessThanSelf: aString
	"string version components are always '<' integer component"
	
	^true! !

!Integer methodsFor: '*metacello-core' stamp: 'dkh 8/13/2009 10:36'!
metacelloVersionComponentLessThan: aMetacelloVersonComponent

	^aMetacelloVersonComponent metacelloIntegerLessThanSelf: self! !

Object subclass: #MetacelloPlatform
	instanceVariableNames: 'bypassProgressBars'
	classVariableNames: 'Current'
	poolDictionaries: ''
	category: 'Metacello-Core-Model'!

!MetacelloPlatform class methodsFor: 'accessing' stamp: 'dkh 7/24/2009 15:21'!
current
	Current
		ifNil: [Current := MetacelloPlatform new].
	^ Current! !

!MetacelloPlatform class methodsFor: 'initialize-release' stamp: 'dkh 7/24/2009 15:29'!
initialize
	"MetacelloPlatform initialize"
	"MetacelloGemStonePlatform initialize"
	"MetacelloPharoPlatform initialize"
	"MetacelloSqueakPlatform initialize"

	Current := self new! !

!MetacelloPlatform methodsFor: 'utilities' stamp: 'dkh 10/28/2009 10:53'!
authorName

	Smalltalk at: #Author ifPresent: [:cl | ^cl perform: #initials ].
	^'no developer initials'! !

!MetacelloPlatform methodsFor: 'accessing' stamp: 'DaleHenrichs 3/11/2010 19:24'!
bypassProgressBars

	bypassProgressBars == nil ifTrue: [ bypassProgressBars := false ].
	^ bypassProgressBars! !

!MetacelloPlatform methodsFor: 'accessing' stamp: 'DaleHenrichs 3/11/2010 19:24'!
bypassProgressBars: anObject
	bypassProgressBars := anObject! !

!MetacelloPlatform methodsFor: 'caching' stamp: 'DaleHenrichs 3/5/2010 10:04'!
clearCurrentVersionCache

	MetacelloClearStackCacheNotification signal! !

!MetacelloPlatform methodsFor: 'notification' stamp: 'DaleHenrichs 3/11/2010 19:26'!
collection: aCollection do: aBlock displaying: aString

	aCollection do: aBlock! !

!MetacelloPlatform methodsFor: 'user interaction' stamp: 'dkh 8/12/2009 17:51'!
confirm: aString
	
	^(Smalltalk hasClassNamed: #UIManager)
		ifTrue: [ (Smalltalk classNamed: #UIManager) default perform: #confirm: with: aString ]
		ifFalse: [
			"throw warning and answer true, if no way to announce"
			Warning signal: aString.
			true ]! !

!MetacelloPlatform methodsFor: 'repository creation' stamp: 'TestRunner 12/22/2009 11:50'!
createRepository: aRepositorySpec

	| type |
	type := aRepositorySpec type.
	type = 'http'
		ifTrue: [^ MCHttpRepository
				location: aRepositorySpec description
				user: aRepositorySpec username
				password: aRepositorySpec password].
	type = 'directory'
		ifTrue: [^ MCDirectoryRepository new
				directory: (FileDirectory on: aRepositorySpec description)].
	type = 'dictionary'
		ifTrue: [	| description headerSize globalName |	
			description := aRepositorySpec description.
			headerSize := 'dictionary://' size.
			globalName := (description copyFrom: headerSize + 1 to: description size) asSymbol.
 			^Smalltalk at: globalName ifAbsent: [Smalltalk at: globalName put: MCDictionaryRepository new ]].
	^ nil! !

!MetacelloPlatform methodsFor: 'attributes' stamp: 'dkh 7/24/2009 15:24'!
defaultPlatformAttributes
	| versionString |
	Smalltalk at: #SystemVersion ifPresent: [:cl |
		versionString := cl current version.
		(versionString beginsWith: 'Pharo')
			ifTrue: [ ^ #(#squeakCommon #pharo ) ].
		(versionString beginsWith: 'Squeak')
			ifTrue: [^ #(#squeakCommon #squeak )]].
	^ #(#gemstone )! !

!MetacelloPlatform methodsFor: 'notification' stamp: 'DaleHenrichs 03/08/2010 13:31'!
do: aBlock displaying: aString

	aBlock value! !

!MetacelloPlatform methodsFor: 'repository creation' stamp: 'tbn 6/9/2010 13:56'!
extractTypeFromDescription: description

	description == nil ifTrue: [ ^nil ].
	((description beginsWith: '/') or: [ description second = $:]) ifTrue: [ ^'directory' ].
	(description beginsWith: 'dictionary://') ifTrue: [ ^'dictionary' ].
	^'http'! !

!MetacelloPlatform methodsFor: 'caching' stamp: 'DaleHenrichs 2/8/2010 12:39'!
stackCacheFor: cacheName at: key doing: aBlock

	self 
		useStackCacheDuring: [:dict | | cache |
			cache := dict at: cacheName ifAbsent: [].
			cache ~~ nil
				ifTrue: [ | value hasEntry |
					hasEntry := true.
					value := cache at: key ifAbsent: [ hasEntry := false ].
					hasEntry ifTrue: [ ^value ]]
				ifFalse: [ 
					cache := Dictionary new.
					dict at: cacheName put: cache ].
			^aBlock value: cache ] 
		defaultDictionary: nil! !

!MetacelloPlatform methodsFor: 'utilities' stamp: 'dkh 10/7/2009 10:51'!
timestamp

	^DateAndTime now printString! !

!MetacelloPlatform methodsFor: 'transactions' stamp: 'dkh 10/23/2009 11:59'!
transact: aBlock
	"On GemStone, we want to optionally abort before command execution and commit after 
	 common execution. Other plaforms don't need to do anything special.
	 Returning out of block, skips commit."
	
	aBlock value! !

!MetacelloPlatform methodsFor: 'caching' stamp: 'DaleHenrichs 2/8/2010 13:01'!
useStackCacheDuring: aBlock defaultDictionary: defaultDictionary

	| dict |
	dict := MetacelloStackCacheNotification signal.
	dict == nil
		ifTrue: [ 
			dict := defaultDictionary == nil
				ifTrue: [ Dictionary new ]
				ifFalse: [ defaultDictionary ]].
	[ ^aBlock value: dict ]
			on: MetacelloStackCacheNotification, MetacelloClearStackCacheNotification
			do: [:ex |  
				(ex isKindOf: MetacelloStackCacheNotification) 
					ifTrue: [ ex resume: dict ].
				(ex isKindOf: MetacelloClearStackCacheNotification)
					ifTrue: [ 
						dict keys do: [:k | | c |
							c := dict at: k.
							c keys do: [:ck | c removeKey: ck ].
							dict removeKey: k ].
						ex resume ]]! !

Object subclass: #MetacelloProject
	instanceVariableNames: 'versionMap loader loaderClass loadType configuration projectAttributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Model'!

!MetacelloProject class methodsFor: 'instance creation' stamp: 'dkh 9/4/2009 01:43'!
new

	| inst |
	inst := self basicNew.
	^inst configuration: inst; yourself! !

!MetacelloProject class methodsFor: 'instance creation' stamp: 'dkh 9/4/2009 01:43'!
on: aConfig

	^self basicNew configuration: aConfig; yourself! !

!MetacelloProject methodsFor: 'private' stamp: 'dkh 10/14/2009 16:33'!
attributes

	^(OrderedCollection with: #common)
		addAll: self platformAttributes;
		yourself! !

!MetacelloProject methodsFor: 'versions' stamp: 'SeanDeNigris 8/17/2010 16:15'!
bleedingEdge

	| coll |
	coll := (self map values asArray sortBy: [:a :b | a <= b ]) asOrderedCollection.
	coll isEmpty ifTrue: [ ^nil ].
	^coll last! !

!MetacelloProject methodsFor: 'accessing' stamp: 'tg 8/30/2009 17:48'!
configuration
	^ configuration! !

!MetacelloProject methodsFor: 'accessing' stamp: 'tg 8/30/2009 17:48'!
configuration: anObject
	configuration := anObject! !

!MetacelloProject methodsFor: 'versions' stamp: 'DaleHenrichs 2/6/2010 09:42'!
currentVersion

	| cacheKey |
	cacheKey := self configuration class.
	^MetacelloPlatform current 
		stackCacheFor: #currentVersion
		at: cacheKey
		doing: [:cache |	|  cv versions |
			cv := nil.
			versions := self sortedAndFilteredVersions.
			versions do: [:vrsn |
				vrsn isAllLoadedToSpec
					ifTrue: [ 
						cv := vrsn copy.
						cv versionStatus: #allLoadedToSpec. "all projects and packages are loaded and match specification"
						^cache at: cacheKey put: cv ]].
			versions do: [:vrsn |
				vrsn isLoadedToSpec
					ifTrue: [ 
						cv := vrsn copy.
						cv versionStatus: #loadedToSpec. "all loaded projects and packages match specifications (at least one package loaded)"
						^cache at: cacheKey put:  cv ]].
			versions do: [:vrsn |
				vrsn isLoadedMatchConstraints
					ifTrue: [ 
						cv := vrsn copy.
						cv versionStatus: #loadedMatchConstraints. "all loaded projects and packages match constraints (at least one package loaded)"
						^cache at: cacheKey put: cv ]].
			versions do: [:vrsn |
				vrsn isSomethingLoaded
					ifTrue: [ 
						cv := vrsn copy.
						cv versionStatus: #somethingLoaded. "at least one package loaded"
						^cache at: cacheKey put: cv ]].
			^cache at: cacheKey put:  nil ].! !

!MetacelloProject methodsFor: 'versions' stamp: 'DaleHenrichs 2/6/2010 09:43'!
currentVersionAgainst: resolvedPackageAndProjectNames

	| cacheKey |
	cacheKey := Array 
			with: self configuration class
			with: (resolvedPackageAndProjectNames sort: [:a :b | a <= b ]).
	^MetacelloPlatform current 
		stackCacheFor: #currentVersionAgainst:
		at: cacheKey
		doing: [:cache |	|  cv versions |
			cv := nil.
			versions := self sortedAndFilteredVersions.
			versions do: [:vrsn |
				(vrsn spec isAllLoadedToSpecAgainst: resolvedPackageAndProjectNames)
					ifTrue: [ 
						cv := vrsn copy.
						cv versionStatus: #allLoadedToSpec. "all projects and packages are loaded and match specification"
						^cache at: cacheKey put: cv ]].
			versions do: [:vrsn |
				(vrsn spec isLoadedToSpecAgainst: resolvedPackageAndProjectNames)
					ifTrue: [ 
						cv := vrsn copy.
						cv versionStatus: #loadedToSpec. "all loaded projects and packages match specifications (at least one package loaded)"
						^cache at: cacheKey put:  cv ]].
			versions do: [:vrsn |
				(vrsn spec isLoadedMatchConstraintsAgainst: resolvedPackageAndProjectNames)
					ifTrue: [ 
						cv := vrsn copy.
						cv versionStatus: #loadedMatchConstraints. "all loaded projects and packages match constraints (at least one package loaded)"
						^cache at: cacheKey put: cv ]].
			versions do: [:vrsn |
				(vrsn spec isSomethingLoadedAgainst: resolvedPackageAndProjectNames)
					ifTrue: [ 
						cv := vrsn copy.
						cv versionStatus: #somethingLoaded. "at least one package loaded"
						^cache at: cacheKey put: cv ]].
			^cache at: cacheKey put:  nil ].! !

!MetacelloProject methodsFor: 'accessing' stamp: 'dkh 6/18/2009 12:52'!
defaultBlessing

	^#release! !

!MetacelloProject methodsFor: 'spec classes' stamp: 'dkh 6/9/2009 16:03'!
defaultLoaderClass

	^MetacelloSpecLoader! !

!MetacelloProject methodsFor: 'private' stamp: 'dkh 7/24/2009 15:37'!
defaultPlatformAttributes

	^ MetacelloPlatform current defaultPlatformAttributes! !

!MetacelloProject methodsFor: 'private' stamp: 'dkh 10/7/2009 10:05'!
excludeFromLatestVersion

	^#(development broken baseline)! !

!MetacelloProject methodsFor: 'spec classes' stamp: 'dkh 10/4/2009 12:17'!
groupSpec

	^self groupSpecClass for: self! !

!MetacelloProject methodsFor: 'spec classes' stamp: 'dkh 10/4/2009 12:18'!
groupSpecClass

	^MetacelloGroupSpec! !

!MetacelloProject methodsFor: 'accessing' stamp: 'dkh 9/10/2009 15:14'!
label

	^self configuration class name! !

!MetacelloProject methodsFor: 'accessing' stamp: 'SeanDeNigris 8/17/2010 16:19'!
lastVersion

	self deprecated: 'Please use #bleedingEdge.'.

	^ self bleedingEdge.! !

!MetacelloProject methodsFor: 'versions' stamp: 'SeanDeNigris 8/17/2010 16:20'!
latestVersion
	
	self deprecated: 'Please use #stableVersion instead.'.

	^ self stableVersion.! !

!MetacelloProject methodsFor: 'versions' stamp: 'dkh 8/26/2009 10:07'!
latestVersion: blessing

	^(self map values select: [:version | blessing = version blessing ]) detectMax: [:version | version ]! !

!MetacelloProject methodsFor: 'versions' stamp: 'dkh 1/5/2010 10:43'!
latestVersionMatching: versionPatternString
	"Answer whether the version number of the receiver matches the given pattern string.

	 A Metacello version number is made up of version sequences delimited by the characters $. and $-.
	 The $. introduces a numeric version sequence and $- introduces an alphanumeric version sequence.
	 
	 A version pattern is made up of version pattern match sequences. also delimited by the characters $. 
	 and $-.. Each pattern match sequence is tested against the corresponding version sequence of the 
	 receiver, using the 'standard' pattern matching rules. All sequences must answer true for a match.
	
	 The special pattern sequence '?' is a match for the corresponding version sequence and all subsequent 
	 version sequences. '?' as the version pattern matches all versions. No more version pattern 
	 sequences are permitted once the '?' sequence is used. If used, it is the last version pattern
	 sequence."

	^self 
		latestVersionMatching: versionPatternString 
		includedBlessings: #()
		excludedBlessings: self excludeFromLatestVersion! !

!MetacelloProject methodsFor: 'versions' stamp: 'dkh 12/21/2009 14:04'!
latestVersionMatching: versionPatternString excludedBlessings: excluded

	^self 
		latestVersionMatching: versionPatternString 
		includedBlessings: #()
		excludedBlessings: excluded! !

!MetacelloProject methodsFor: 'versions' stamp: 'dkh 12/21/2009 13:59'!
latestVersionMatching: versionPatternString includedBlessings: included

	^self 
		latestVersionMatching: versionPatternString 
		includedBlessings: included
		excludedBlessings: self excludeFromLatestVersion! !

!MetacelloProject methodsFor: 'versions' stamp: 'dkh 12/21/2009 14:41'!
latestVersionMatching: versionPatternString includedBlessings: included excludedBlessings: excludedBlessings

	| excluded |
	excluded := excludedBlessings asSet copy.
	excluded removeAllFoundIn: included.
	^(self map values select: [:version |
		(included isEmpty or: [ included includes: version blessing ]) 
			and: [ (excluded includes: version blessing) not 
				and: [ version versionNumber match: versionPatternString ]]])
		detectMax: [:version | version ]! !

!MetacelloProject methodsFor: 'loading' stamp: 'dkh 9/10/2009 15:09'!
load: aVersionString

	^(self version: aVersionString) load! !

!MetacelloProject methodsFor: 'spec classes' stamp: 'dkh 6/18/2009 10:10'!
loader

	^loader! !

!MetacelloProject methodsFor: 'spec classes' stamp: 'dkh 6/18/2009 10:10'!
loader: aLoader

	loader := aLoader! !

!MetacelloProject methodsFor: 'spec classes' stamp: 'dkh 6/9/2009 16:04'!
loaderClass

	loaderClass == nil ifTrue: [ loaderClass := self defaultLoaderClass ].
	^loaderClass! !

!MetacelloProject methodsFor: 'spec classes' stamp: 'dkh 6/9/2009 16:05'!
loaderClass: aMetacelloSpecLoader

	loaderClass := aMetacelloSpecLoader! !

!MetacelloProject methodsFor: 'accessing' stamp: 'dkh 9/10/2009 13:27'!
loadType
	"#atomic or #linear"

	loadType == nil ifTrue: [ ^#atomic ].
	^loadType! !

!MetacelloProject methodsFor: 'accessing' stamp: 'dkh 9/10/2009 13:27'!
loadType: aSymbol
	"#atomic or #linear"

	loadType := aSymbol! !

!MetacelloProject methodsFor: 'accessing' stamp: 'dkh 10/28/2009 10:52'!
map

	^versionMap! !

!MetacelloProject methodsFor: 'accessing' stamp: 'dkh 10/3/2009 17:31'!
map: aDictionary

	versionMap := aDictionary! !

!MetacelloProject methodsFor: 'spec classes' stamp: 'dkh 10/4/2009 12:18'!
packagesSpec

	^self packagesSpecClass for: self! !

!MetacelloProject methodsFor: 'spec classes' stamp: 'dkh 10/4/2009 12:18'!
packagesSpecClass

	^MetacelloPackagesSpec! !

!MetacelloProject methodsFor: 'private' stamp: 'dkh 10/17/2009 09:32'!
platformAttributes

	 ^self projectPlatformAttributes! !

!MetacelloProject methodsFor: 'private' stamp: 'dkh 7/28/2009 16:42'!
pragmaKeywords

	^#(version:attribute: blessing:attribute: description:attribute: required:attribute: groups:attribute: doits:attribute:)! !

!MetacelloProject methodsFor: 'printing' stamp: 'dkh 10/26/2009 16:07'!
printOn: aStream

	self configuration class printOn: aStream.
	aStream nextPut: $(.
	self versions do: [:vrsn | aStream nextPutAll: vrsn printString; nextPut: $,; space ].
	aStream 
		nextPut: $).! !

!MetacelloProject methodsFor: 'accessing' stamp: 'dkh 9/2/2009 06:02'!
project

	^self! !

!MetacelloProject methodsFor: 'private' stamp: 'dkh 9/2/2009 08:31'!
projectAttributes

	projectAttributes ~~ nil ifTrue: [ ^projectAttributes ].
	^#()! !

!MetacelloProject methodsFor: 'private' stamp: 'dkh 9/2/2009 08:31'!
projectAttributes: aList

	projectAttributes := aList! !

!MetacelloProject methodsFor: 'private' stamp: 'dkh 9/2/2009 07:38'!
projectPlatformAttributes

	| list aBlock |
	list := OrderedCollection new.
	(aBlock := self projectAttributes) ~~ nil 
		ifTrue: [ list addAll: aBlock value ].
	^self defaultPlatformAttributes, list! !

!MetacelloProject methodsFor: 'spec classes' stamp: 'dkh 10/4/2009 12:18'!
projectReferenceSpec

	^self projectReferenceSpecClass for: self! !

!MetacelloProject methodsFor: 'spec classes' stamp: 'dkh 10/4/2009 12:18'!
projectReferenceSpecClass

	^MetacelloProjectReferenceSpec! !

!MetacelloProject methodsFor: 'spec classes' stamp: 'dkh 06/01/2009 14:54'!
projectSpec

	^self projectSpecClass for: self! !

!MetacelloProject methodsFor: 'spec classes' stamp: 'dkh 10/4/2009 12:18'!
projectSpecClass

	^MetacelloMCProjectSpec! !

!MetacelloProject methodsFor: 'private' stamp: 'DaleHenrichs 1/19/2010 15:31'!
sortedAndFilteredVersions

		^(self map values asArray sortBy: [:a :b | a >= b ]) select: [:vrsn | (#(broken baseline) includes: vrsn blessing) not ].! !

!MetacelloProject methodsFor: 'versions' stamp: 'SeanDeNigris 8/17/2010 16:13'!
stableVersion

	| excluded |
	excluded := self excludeFromLatestVersion.
	^(self map values select: [:version |  
		(excluded includes: version blessing) not ]) detectMax: [:version | version ]! !

!MetacelloProject methodsFor: 'spec classes' stamp: 'dkh 6/5/2009 11:47'!
valueHolderSpec

	^self valueHolderSpecClass for: self! !

!MetacelloProject methodsFor: 'spec classes' stamp: 'dkh 6/5/2009 11:47'!
valueHolderSpecClass

	^MetacelloValueHolderSpec! !

!MetacelloProject methodsFor: 'versions' stamp: 'dkh 8/9/2009 15:11'!
version: aVersionString

	^self version: aVersionString ifAbsent: [ ^nil ]! !

!MetacelloProject methodsFor: 'versions' stamp: 'dkh 06/26/2009 09:28'!
version: aVersionString ifAbsent: aBlock

	^self map at: aVersionString  ifAbsent: aBlock! !

!MetacelloProject methodsFor: 'versions' stamp: 'dkh 9/10/2009 15:07'!
versions

	^self map values asArray sortBy: [:a :b | a <= b ]! !

!MetacelloProject methodsFor: 'spec classes' stamp: 'dkh 06/01/2009 11:00'!
versionSpec

	^self versionSpecClass for: self! !

!MetacelloProject methodsFor: 'spec classes' stamp: 'dkh 6/4/2009 18:51'!
versionSpecClass

	^MetacelloVersionSpec! !

Object subclass: #MetacelloSpec
	instanceVariableNames: 'project loader'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Specs'!

MetacelloSpec subclass: #MetacelloAbstractPackageSpec
	instanceVariableNames: 'name requires includes answers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Specs'!

!MetacelloAbstractPackageSpec methodsFor: 'adding' stamp: 'dkh 10/5/2009 12:07'!
addToMetacelloPackages: aMetacelloPackagesSpec

	aMetacelloPackagesSpec addMember: 
		(aMetacelloPackagesSpec addMember 
			name: self name;
			spec: self;
			yourself)! !

!MetacelloAbstractPackageSpec methodsFor: 'querying' stamp: 'dkh 1/4/2010 18:54'!
answers

	answers == nil ifTrue: [ answers := #() ].
	^answers! !

!MetacelloAbstractPackageSpec methodsFor: 'accessing' stamp: 'dkh 1/4/2010 19:01'!
answers: aListOfPairs

	self setAnswers: aListOfPairs! !

!MetacelloAbstractPackageSpec methodsFor: 'printing' stamp: 'dkh 01/07/2010 16:48'!
configMethodBodyOn: aStream hasName: hasName cascading: cascading indent: indent

	| hasCascading hasRequires hasIncludes hasAnswers |
	hasCascading := cascading.
	hasRequires := self requires isEmpty not.
	hasIncludes := self includes isEmpty not.
	hasAnswers := self answers isEmpty not.
	hasRequires
		ifTrue: [ 
			hasName | hasCascading
				ifTrue: [ aStream cr; tab: indent ].
			aStream nextPutAll: 'requires: #('.
			self requires do: [:str | aStream nextPutAll: str printString, ' ' ].
			hasIncludes | hasAnswers | hasCascading
				ifTrue: [ aStream nextPutAll: ');' ]
				ifFalse: [ aStream nextPut: $) ]].
	hasIncludes
		ifTrue: [ 
			hasName | hasRequires | hasCascading
				ifTrue: [ aStream cr; tab: indent ].
			aStream nextPutAll: 'includes: #('.
			self includes do: [:str | aStream nextPutAll: str printString, ' ' ].
			hasAnswers | hasCascading
				ifTrue: [ aStream nextPutAll: ');' ]
				ifFalse: [ aStream nextPut: $) ]].
	hasAnswers
		ifTrue: [ 
			hasName | hasRequires | hasIncludes | hasCascading
				ifTrue: [ aStream cr; tab: indent ].
			aStream nextPutAll: 'supplyingAnswers #( '.
			self answers do: [:ar | 
				aStream nextPutAll: '#( '.
				ar do: [:val | 
					(val isString or: [ val isNumber or: [ val isSymbol or: [ val isCharacter ]]])
						ifTrue: [  aStream nextPutAll: val printString, ' ' ].
					val == true
						ifTrue: [  aStream nextPutAll: 'true ' ].
					val == false
						ifTrue: [  aStream nextPutAll: 'false ' ]].
				aStream nextPutAll: ') ' ].
			hasCascading
				ifTrue: [ aStream nextPutAll: ');' ]
				ifFalse: [ aStream nextPut: $) ]].! !

!MetacelloAbstractPackageSpec methodsFor: 'printing' stamp: 'dkh 10/5/2009 12:41'!
configMethodCascadeOn: aStream member: aMember last: lastCascade indent: indent

	self subclassResponsibility! !

!MetacelloAbstractPackageSpec methodsFor: 'printing' stamp: 'DaleHenrichs 1/21/2010 16:01'!
configMethodOn: aStream for: aValue selector: selector cascading: cascading cascade: cascade indent: indent

	aValue == nil ifTrue: [ ^self ].
	cascading ifTrue: [ aStream cr; tab: indent ].
	aStream  nextPutAll: selector, aValue printString.
	cascade ifTrue: [ aStream nextPut: $; ]! !

!MetacelloAbstractPackageSpec methodsFor: 'querying' stamp: 'dkh 9/23/2009 08:45'!
includes

	includes == nil ifTrue: [ includes := #() ].
	^includes! !

!MetacelloAbstractPackageSpec methodsFor: 'accessing' stamp: 'dkh 9/23/2009 08:45'!
includes: aCollection

	aCollection setIncludesInMetacelloPackage: self! !

!MetacelloAbstractPackageSpec methodsFor: 'private' stamp: 'dkh 10/10/2009 10:59'!
includesForPackageOrdering

	^#()! !

!MetacelloAbstractPackageSpec methodsFor: 'printing' stamp: 'DaleHenrichs 3/9/2010 16:33'!
label

	^self name! !

!MetacelloAbstractPackageSpec methodsFor: 'merging' stamp: 'dkh 6/23/2009 11:22'!
mergeIntoMetacelloPackages: aMetacelloPackagesSpec

	aMetacelloPackagesSpec addMember: 
		(aMetacelloPackagesSpec mergeMember 
			name: self name;
			spec: self;
			yourself)! !

!MetacelloAbstractPackageSpec methodsFor: 'merging' stamp: 'DaleHenrichs 1/21/2010 20:36'!
mergeMap

	| map |
	map := super mergeMap.
	map at: #requires put: requires.
	map at: #includes put: includes.
	map at: #answers put: answers.
	^map! !

!MetacelloAbstractPackageSpec methodsFor: 'merging' stamp: 'DaleHenrichs 1/21/2010 20:21'!
mergeSpec: anotherSpec

	| newSpec map anotherRequires anotherIncludes anotherAnswers |
	newSpec := super mergeSpec: anotherSpec.
	map := anotherSpec mergeMap.
	anotherSpec name ~~ nil 
		ifTrue: [ newSpec name: anotherSpec name ].
	(anotherRequires := map at: #requires) ~~ nil
		ifTrue: [ newSpec setRequires: self requires, anotherRequires ].
	(anotherIncludes := map at: #includes) ~~ nil
		ifTrue: [ newSpec setIncludes: self includes, anotherIncludes ].
	(anotherAnswers := map at: #answers) ~~ nil
		ifTrue: [ newSpec setAnswers: self answers, anotherAnswers ].
	^newSpec! !

!MetacelloAbstractPackageSpec methodsFor: 'querying' stamp: 'dkh 6/23/2009 11:24'!
name

	^name! !

!MetacelloAbstractPackageSpec methodsFor: 'accessing' stamp: 'dkh 6/23/2009 11:24'!
name: aString

	name := aString! !

!MetacelloAbstractPackageSpec methodsFor: 'merging' stamp: 'DaleHenrichs 1/21/2010 20:21'!
nonOverridable

	^#( includes requires answers )! !

!MetacelloAbstractPackageSpec methodsFor: 'copying' stamp: 'dkh 1/4/2010 18:55'!
postCopy

	super postCopy.
	requires := requires copy.
	includes := includes copy.
	answers := answers copy.! !

!MetacelloAbstractPackageSpec methodsFor: 'visiting' stamp: 'dkh 10/5/2009 09:38'!
projectDo: projectBlock packageDo: packageBlock groupDo: groupBlock

	self subclassResponsibility! !

!MetacelloAbstractPackageSpec methodsFor: 'accessing' stamp: 'dkh 10/24/2009 19:41'!
referencedSpec

	^self! !

!MetacelloAbstractPackageSpec methodsFor: 'removing' stamp: 'dkh 6/23/2009 11:24'!
removeFromMetacelloPackages: aMetacelloPackagesSpec

	aMetacelloPackagesSpec addMember: 
		(aMetacelloPackagesSpec removeMember 
			name: self name;
			spec: self;
			yourself)! !

!MetacelloAbstractPackageSpec methodsFor: 'querying' stamp: 'dkh 6/23/2009 11:25'!
requires

	requires == nil ifTrue: [ requires := #() ].
	^requires! !

!MetacelloAbstractPackageSpec methodsFor: 'accessing' stamp: 'dkh 6/23/2009 11:25'!
requires: aCollection

	aCollection setRequiresInMetacelloPackage: self! !

!MetacelloAbstractPackageSpec methodsFor: 'private' stamp: 'dkh 1/4/2010 18:54'!
setAnswers: aCollection

	answers := aCollection! !

!MetacelloAbstractPackageSpec methodsFor: 'private' stamp: 'dkh 9/23/2009 08:45'!
setIncludes: aCollection

	includes := aCollection! !

!MetacelloAbstractPackageSpec methodsFor: 'private' stamp: 'dkh 6/23/2009 11:25'!
setRequires: aCollection

	requires := aCollection! !

MetacelloAbstractPackageSpec subclass: #MetacelloGroupSpec
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Specs'!

!MetacelloGroupSpec methodsFor: 'accessing' stamp: 'dkh 1/4/2010 19:02'!
answers: aListOfPairs

	self shouldNotImplement! !

!MetacelloGroupSpec methodsFor: 'printing' stamp: 'dkh 10/24/2009 15:09'!
configMethodCascadeOn: aStream member: aMember last: lastCascade indent: indent

	aMember methodUpdateSelector == #remove:
		ifTrue: [ aStream nextPutAll: 'removeGroup: ', self name printString. ]
		ifFalse: [
			aStream 
				nextPutAll: 'group: ', self name printString;
				space; 
				nextPutAll: aMember methodUpdateSelector asString, ' #('.
			self includes do: [:str | aStream nextPutAll: str printString, ' ' ].
			aStream nextPut: $) ].
	lastCascade
		ifTrue: [ aStream nextPut: $. ]
		ifFalse: [ aStream nextPut: $;; cr ].! !

!MetacelloGroupSpec methodsFor: 'printing' stamp: 'dkh 1/6/2010 18:39'!
configMethodOn: aStream indent: indent

	aStream 
		tab: indent;
		nextPutAll: 'spec '; cr;
		tab: indent + 1;
		nextPutAll: 'name: ', self name printString, ';'.
	self configMethodBodyOn: aStream hasName: true cascading: false indent: indent + 1.
	aStream nextPut: $.! !

!MetacelloGroupSpec methodsFor: 'visiting' stamp: 'dkh 10/5/2009 09:39'!
projectDo: projectBlock packageDo: packageBlock groupDo: groupBlock

	groupBlock value: self! !

!MetacelloGroupSpec methodsFor: 'accessing' stamp: 'dkh 10/4/2009 09:41'!
requires: aCollection

	self shouldNotImplement! !

MetacelloAbstractPackageSpec subclass: #MetacelloProjectReferenceSpec
	instanceVariableNames: 'projectReference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Specs'!

!MetacelloProjectReferenceSpec methodsFor: 'accessing' stamp: 'dkh 1/4/2010 19:02'!
answers: aListOfPairs

	self shouldNotImplement! !

!MetacelloProjectReferenceSpec methodsFor: 'printing' stamp: 'dkh 10/7/2009 16:35'!
configMethodCascadeOn: aStream member: aMember last: lastCascade indent: indent

	aMember methodUpdateSelector  == #remove:
		ifTrue: [ aStream  nextPutAll: 'removeProject: ', self name printString ]
		ifFalse: [
			self projectReference == nil ifTrue: [ ^self ].
			aStream nextPutAll: 'project: ', self projectName printString, ' '.
			aMember methodUpdateSelector == #copy:
				ifTrue: [ aStream nextPutAll: 'copyFrom: ', aMember sourceName printString, ' ' ].
			self projectReference configShortCutMethodOn: aStream member: aMember indent: indent + 1 ].
	lastCascade
		ifTrue: [ aStream nextPut: $. ]
		ifFalse: [ aStream nextPut: $;; cr ].! !

!MetacelloProjectReferenceSpec methodsFor: 'printing' stamp: 'dkh 10/24/2009 19:53'!
configMethodOn: aStream indent: indent

	aStream 
		tab: indent; nextPutAll: 'spec '; cr;
		tab: indent + 1; nextPutAll: 'name: ', self name printString; nextPut: $;; cr;
		tab: indent + 1; nextPutAll: 'projectReference: '; nextPut: $[; cr.
	aStream 
		tab: indent + 2; nextPutAll: 'spec'.
	self projectReference configMethodBodyOn: aStream indent: indent + 2.
	aStream nextPutAll: ' ].'! !

!MetacelloProjectReferenceSpec methodsFor: 'accessing' stamp: 'dkh 10/5/2009 10:13'!
includes: aCollection

	self shouldNotImplement! !

!MetacelloProjectReferenceSpec methodsFor: 'merging' stamp: 'dkh 10/5/2009 11:31'!
mergeMap

	| map |
	map := super mergeMap.
	map at: #projectReference put: projectReference.
	^map! !

!MetacelloProjectReferenceSpec methodsFor: 'merging' stamp: 'DaleHenrichs 1/21/2010 20:00'!
mergeSpec: anotherSpec

	| newSpec map anotherProjectReference |
	newSpec := super mergeSpec: anotherSpec.
	map := anotherSpec mergeMap.
	(anotherProjectReference := map at: #projectReference) ~~ nil
		ifTrue: [ 
			newSpec projectReference: 
				(newSpec projectReference == nil
					ifTrue: [ anotherProjectReference ]
					ifFalse: [ newSpec projectReference mergeSpec: anotherProjectReference ])].
	^newSpec! !

!MetacelloProjectReferenceSpec methodsFor: 'merging' stamp: 'dkh 10/8/2009 20:32'!
nonOverridable

	^super nonOverridable, #( projectReference )! !

!MetacelloProjectReferenceSpec methodsFor: 'visiting' stamp: 'dkh 10/24/2009 19:43'!
projectDo: projectBlock packageDo: packageBlock groupDo: groupBlock

	projectBlock value: self! !

!MetacelloProjectReferenceSpec methodsFor: 'accessing' stamp: 'dkh 9/23/2009 10:51'!
projectName
	^self projectReference name! !

!MetacelloProjectReferenceSpec methodsFor: 'querying' stamp: 'dkh 9/23/2009 10:51'!
projectReference
	^ projectReference! !

!MetacelloProjectReferenceSpec methodsFor: 'accessing' stamp: 'dkh 9/23/2009 10:51'!
projectReference: anObject
	projectReference := anObject! !

!MetacelloProjectReferenceSpec methodsFor: 'accessing' stamp: 'dkh 10/24/2009 19:42'!
referencedSpec

	^self projectReference! !

!MetacelloProjectReferenceSpec methodsFor: 'querying' stamp: 'DaleHenrichs 2/5/2010 13:42'!
relativeCurrentVersion

	^self projectReference relativeCurrentVersion! !

!MetacelloProjectReferenceSpec methodsFor: 'accessing' stamp: 'dkh 10/4/2009 11:40'!
requires: aCollection

	self shouldNotImplement! !

!MetacelloProjectReferenceSpec methodsFor: 'querying' stamp: 'dkh 10/24/2009 10:38'!
version

	^self projectReference version! !

MetacelloSpec subclass: #MetacelloMemberListSpec
	instanceVariableNames: 'list'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Members'!

!MetacelloMemberListSpec methodsFor: 'actions' stamp: 'dkh 10/4/2009 11:22'!
add: aSpec

	self subclassResponsibility! !

!MetacelloMemberListSpec methodsFor: 'adding' stamp: 'dkh 10/4/2009 10:38'!
addMember: aMember

	self list add: aMember! !

!MetacelloMemberListSpec methodsFor: 'actions' stamp: 'dkh 10/4/2009 10:59'!
copy: aMemberSpec

	self addMember: aMemberSpec! !

!MetacelloMemberListSpec methodsFor: 'actions' stamp: 'dkh 10/4/2009 11:23'!
copy: specNamed to: aSpec

	self subclassResponsibility! !

!MetacelloMemberListSpec methodsFor: 'enumeration' stamp: 'dkh 6/12/2009 13:38'!
do: aBlock

	self map values do: aBlock! !

!MetacelloMemberListSpec methodsFor: 'testing' stamp: 'dkh 6/17/2009 16:12'!
isEmpty

	^self list isEmpty! !

!MetacelloMemberListSpec methodsFor: 'accessing' stamp: 'dkh 06/02/2009 18:51'!
list

	list == nil ifTrue: [ list := OrderedCollection new ].
	^list! !

!MetacelloMemberListSpec methodsFor: 'accessing' stamp: 'dkh 06/02/2009 19:10'!
list: aCollection

	list := aCollection! !

!MetacelloMemberListSpec methodsFor: 'accessing' stamp: 'dkh 10/4/2009 10:20'!
map
	
	| map |
	map := Dictionary new.
	self list do: [:member | 
		member
			applyAdd: [:memberSpec | self mapAdd: memberSpec into: map ] 
			copy: [:memberSpec | self mapCopy: memberSpec into: map ]
			merge: [:memberSpec | self mapMerge: memberSpec into: map ] 
			remove: [:memberSpec | self mapRemove: memberSpec into: map ]].
	^map! !

!MetacelloMemberListSpec methodsFor: 'private' stamp: 'dkh 06/02/2009 19:33'!
mapAdd: aMemberSpec into: map
	
	map at: aMemberSpec name put: aMemberSpec spec! !

!MetacelloMemberListSpec methodsFor: 'private' stamp: 'dkh 10/29/2009 15:43'!
mapCopy: aMemberSpec into: map
	| spec |
	spec :=  map at: aMemberSpec sourceName ifAbsent: [].
	spec == nil
		ifTrue: [ ^self error: 'Source spec named ',  aMemberSpec sourceName printString, ' not found'. ]
		ifFalse: [ map at: aMemberSpec name put: (spec copy mergeSpec: aMemberSpec spec)]! !

!MetacelloMemberListSpec methodsFor: 'private' stamp: 'dkh 6/5/2009 16:08'!
mapMerge: aMemberSpec into: map
	| spec |
	spec :=  map at: aMemberSpec name ifAbsent: [].
	spec == nil
		ifTrue: [ map at: aMemberSpec name put: aMemberSpec spec copy ]
		ifFalse: [ map at: aMemberSpec name put: (spec mergeSpec: aMemberSpec spec)]! !

!MetacelloMemberListSpec methodsFor: 'private' stamp: 'dkh 06/02/2009 19:58'!
mapRemove: aMemberSpec into: map
	
	map removeKey: aMemberSpec name ifAbsent: []! !

!MetacelloMemberListSpec methodsFor: 'actions' stamp: 'dkh 10/4/2009 11:23'!
merge: aSpec

	self subclassResponsibility! !

!MetacelloMemberListSpec methodsFor: 'merging' stamp: 'dkh 06/02/2009 19:26'!
mergeSpec: anotherSpec

	| newSpec val |
	newSpec := super mergeSpec: anotherSpec.
	newSpec list: self list copy.
	anotherSpec list do: [:groupMember | groupMember applyToList: newSpec ].
	^newSpec! !

!MetacelloMemberListSpec methodsFor: 'testing' stamp: 'Dalehenrichs 02/10/2010 10:50'!
notEmpty

	^self list notEmpty! !

!MetacelloMemberListSpec methodsFor: 'copying' stamp: 'dkh 8/9/2009 18:05'!
postCopy

	super postCopy.
	list := list copy! !

!MetacelloMemberListSpec methodsFor: 'actions' stamp: 'dkh 10/4/2009 11:23'!
remove: aSpec

	self subclassResponsibility! !

MetacelloMemberListSpec subclass: #MetacelloPackagesSpec
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Specs'!

!MetacelloPackagesSpec methodsFor: 'actions' stamp: 'dkh 10/4/2009 11:23'!
add: aSpec

	aSpec addToMetacelloPackages: self! !

!MetacelloPackagesSpec methodsFor: 'accessing' stamp: 'dkh 11/12/2009 15:29'!
applyIncludesTo: orderedSpecs for: pkgSpec 
	| movedSpecs baseIndex includedSpec result |
	movedSpecs := Set new.
	baseIndex := orderedSpecs indexOf: pkgSpec.
	pkgSpec includesForPackageOrdering
		do: [:includedSpecName | 
			includedSpec := orderedSpecs
						detect: [:spec | spec name = includedSpecName ]
						ifNone: [].
			(self slideIn: orderedSpecs spec: includedSpec baseIndex: baseIndex seen: IdentitySet new)
				ifTrue: [ movedSpecs add: includedSpec name ]].
	^ movedSpecs! !

!MetacelloPackagesSpec methodsFor: 'printing' stamp: 'dkh 10/22/2009 10:33'!
configMethodOn: aStream indent: indent

	| packageSpecs |
	packageSpecs := self map values.
	packageSpecs size = 0 ifTrue: [ ^aStream nextPutAll: 'spec add: []' ].
	packageSpecs size = 1
		ifTrue: [
			aStream 
				tab: indent; 
				nextPutAll: 'spec add: ['; cr.
			packageSpecs first configMethodOn: aStream indent: indent + 1.
			aStream nextPut: $]; cr ]
		ifFalse: [
			aStream 
				tab: indent; 
				nextPutAll: 'spec'.
			1 to: packageSpecs size do: [:index | | packageSpec |
				packageSpec := packageSpecs at: index.
				aStream 
					tab: indent + 1;
					nextPutAll: 'add: ['; cr.
				packageSpec configMethodOn: aStream indent: indent + 2.
				aStream nextPut: $].
				index < packageSpecs size
					ifTrue: [ aStream nextPut: $; ].
				aStream cr ]]! !

!MetacelloPackagesSpec methodsFor: 'actions' stamp: 'dkh 10/4/2009 11:22'!
copy: specNamed to: spec

	self addMember: 
		(self copyMember 
			name: spec name;
			sourceName: specNamed;
			spec: spec;
			yourself)! !

!MetacelloPackagesSpec methodsFor: 'actions' stamp: 'dkh 10/4/2009 11:23'!
merge: aSpec

	aSpec mergeIntoMetacelloPackages: self! !

!MetacelloPackagesSpec methodsFor: 'accessing' stamp: 'dkh 10/9/2009 13:01'!
packageNamed: aString ifAbsent: aBlock

	^self map at: aString ifAbsent: aBlock! !

!MetacelloPackagesSpec methodsFor: 'accessing' stamp: 'dkh 11/12/2009 15:30'!
packageSpecsInLoadOrder

	| orderedSpecs moved lastMovedSpecs count terminationLimit map specsWithIncludes |
	"specification order is the default order"
	map := self map.
	orderedSpecs := OrderedCollection new.
	self list do: [:member | | spec |
		spec := map at: member name ifAbsent: [].
		(spec == nil or: [ orderedSpecs includes: spec ])
			ifFalse: [ orderedSpecs add: spec ]].
	orderedSpecs isEmpty ifTrue: [ ^orderedSpecs ].
	moved := true.
	count := 0.
	terminationLimit := orderedSpecs size * 2.
	[ moved  ] whileTrue: [
		count := count + 1.
		count > terminationLimit 
			ifTrue: [
				"Cheap termination hack - an APPARENT loop"
				self error: 'Apparent loop in before/after dependency definitions' ]. 
		moved := false.
		orderedSpecs do: [:packageSpec | 
			moved := moved or: [ self sortPackageSpecs: orderedSpecs for: packageSpec ]]].
	lastMovedSpecs := Set new.
	moved := true.
	count := 0.
	specsWithIncludes := orderedSpecs select: [:pkgSpec | pkgSpec includesForPackageOrdering isEmpty not].
	[ moved ] whileTrue: [ | result |
		count := count + 1.
		"count > terminationLimit"
		count > 7
			ifTrue: [
				"Cheap termination hack - an APPARENT loop"
				self error: 'Apparent loop in before/after dependency definitions' ]. 
		moved := false.
		result := Set new.
		specsWithIncludes do: [:packageSpec |
			result addAll: (self applyIncludesTo: orderedSpecs for: packageSpec) ].
		result size = lastMovedSpecs size
			ifTrue: [
				result do: [:name | (lastMovedSpecs includes: name) ifFalse: [ moved := true ]]]
			ifFalse: [ moved := true ].
		lastMovedSpecs := result ].
	^orderedSpecs! !

!MetacelloPackagesSpec methodsFor: 'actions' stamp: 'dkh 10/4/2009 11:23'!
remove: aSpec

	aSpec removeFromMetacelloPackages: self! !

!MetacelloPackagesSpec methodsFor: 'accessing' stamp: 'dkh 11/12/2009 15:29'!
slideIn: orderedSpecs spec: targetSpec baseIndex: baseIndex seen: seen
	| targetIndex requires targetRequires targetRequiresIndexes minIndex baseSpec required |

	(seen includes: targetSpec) ifTrue: [ ^false ].
	targetIndex := orderedSpecs indexOf: targetSpec.
	baseIndex >= targetIndex ifTrue: [ ^false ].
	required := false.
	baseSpec := orderedSpecs at: baseIndex.
	baseIndex + 1 to: targetIndex - 1 do: [:index | | spec |
		spec := orderedSpecs at: index.
		(spec requires includes: baseSpec name) ifTrue: [ required := true ]].
	required ifFalse: [ ^false ].
	requires := targetSpec requires.
	targetRequires := orderedSpecs
				select: [:spec | requires includes: spec name].
	targetRequiresIndexes := targetRequires
				collect: [:spec | orderedSpecs indexOf: spec].
	targetRequiresIndexes add: baseIndex.
	minIndex := targetRequiresIndexes
				detectMax: [:each | each].
	minIndex + 1 < targetIndex
		ifTrue: [
			orderedSpecs remove: targetSpec.
			orderedSpecs add: targetSpec afterIndex: minIndex.
			seen add: targetSpec ]
		ifFalse: [ ^self slideIn: orderedSpecs spec: (orderedSpecs at: minIndex) baseIndex: 1 seen: seen ].
	^true! !

!MetacelloPackagesSpec methodsFor: 'private' stamp: 'dkh 10/10/2009 10:14'!
sortPackageSpecs: orderedSpecs for: packageSpec

	| packageIndex moved movePackage targetPackage targetIndex targetPackages |
	packageIndex := orderedSpecs indexOf: packageSpec.
	moved := movePackage := false.
	targetPackages := packageSpec requires.
	targetPackages do: [:targetPackageName |
		targetPackage := orderedSpecs 
			detect: [:each | each name = targetPackageName ] 
			ifNone: [].
		targetIndex := orderedSpecs indexOf: targetPackage.
		movePackage := movePackage or: [ packageIndex <= targetIndex ]].
	movePackage 
		ifTrue: [
			moved := true.
			orderedSpecs remove: packageSpec ifAbsent: [ ^self error: 'unexpected error removing package' ].
			targetIndex := 0.
			targetPackages do: [:targetPackageName | 
				(targetPackage := orderedSpecs detect: [:each | 
					each name = targetPackageName ] ifNone: []) ~~ nil
						ifTrue: [ targetIndex := targetIndex max: (orderedSpecs indexOf: targetPackage) ]].
			targetIndex == 0
				ifTrue: [ orderedSpecs add: packageSpec beforeIndex: packageIndex ]
				ifFalse: [ orderedSpecs add: packageSpec afterIndex: targetIndex ]].
	^moved! !

MetacelloSpec subclass: #MetacelloMemberSpec
	instanceVariableNames: 'name spec'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Members'!

MetacelloMemberSpec subclass: #MetacelloAddMemberSpec
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Members'!

!MetacelloAddMemberSpec methodsFor: 'actions' stamp: 'dkh 10/4/2009 10:16'!
applyAdd: addBlock copy: copyBlock merge: mergeBlock remove: removeBlock

	addBlock value: self! !

!MetacelloAddMemberSpec methodsFor: 'actions' stamp: 'dkh 06/02/2009 18:26'!
applyToList: aListSpec

	aListSpec add: self! !

!MetacelloAddMemberSpec methodsFor: 'accessing' stamp: 'dkh 10/9/2009 11:43'!
methodUpdateSelector

	^#overrides:! !

MetacelloMemberSpec subclass: #MetacelloCopyMemberSpec
	instanceVariableNames: 'sourceName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Members'!

!MetacelloCopyMemberSpec methodsFor: 'actions' stamp: 'dkh 10/4/2009 10:18'!
applyAdd: addBlock copy: copyBlock merge: mergeBlock remove: removeBlock

	copyBlock value: self! !

!MetacelloCopyMemberSpec methodsFor: 'actions' stamp: 'dkh 10/4/2009 10:19'!
applyToList: aListSpec

	aListSpec copy: self! !

!MetacelloCopyMemberSpec methodsFor: 'accessing' stamp: 'dkh 10/4/2009 10:19'!
methodUpdateSelector

	^#copy:! !

!MetacelloCopyMemberSpec methodsFor: 'accessing' stamp: 'dkh 10/4/2009 10:35'!
sourceName

	^sourceName! !

!MetacelloCopyMemberSpec methodsFor: 'accessing' stamp: 'dkh 10/4/2009 10:36'!
sourceName: aString

	sourceName := aString! !

!MetacelloMemberSpec methodsFor: 'actions' stamp: 'dkh 10/4/2009 10:15'!
applyAdd: addBlock copy: copyBlock merge: mergeBlock remove: removeBlock

	self subclassResponsibility! !

!MetacelloMemberSpec methodsFor: 'actions' stamp: 'dkh 06/02/2009 18:25'!
applyToList: aListSpec

	self subclassResponsibility! !

!MetacelloMemberSpec methodsFor: 'printing' stamp: 'dkh 10/5/2009 12:39'!
configMethodCascadeOn: aStream last: lastCascade indent: indent

	self spec 
		configMethodCascadeOn: aStream 
		member: self 
		last: lastCascade 
		indent: indent! !

!MetacelloMemberSpec methodsFor: 'printing' stamp: 'dkh 10/26/2009 13:34'!
configMethodOn: aStream indent: indent

	aStream nextPutAll: self class name asString, ' member: ('.
	self spec configMethodOn: aStream indent: indent.
	aStream nextPutAll: ')'.! !

!MetacelloMemberSpec methodsFor: 'accessing' stamp: 'dkh 6/15/2009 10:27'!
methodUpdateSelector

	^self subclassResponsibility! !

!MetacelloMemberSpec methodsFor: 'accessing' stamp: 'dkh 06/02/2009 16:36'!
name

	^name! !

!MetacelloMemberSpec methodsFor: 'accessing' stamp: 'dkh 06/02/2009 16:36'!
name: aString

	name := aString! !

!MetacelloMemberSpec methodsFor: 'accessing' stamp: 'dkh 10/4/2009 10:35'!
sourceName

	^self name! !

!MetacelloMemberSpec methodsFor: 'accessing' stamp: 'dkh 06/02/2009 16:50'!
spec

	^spec! !

!MetacelloMemberSpec methodsFor: 'accessing' stamp: 'dkh 06/02/2009 16:50'!
spec: aMetacelloSpec

	spec := aMetacelloSpec! !

MetacelloMemberSpec subclass: #MetacelloMergeMemberSpec
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Members'!

!MetacelloMergeMemberSpec methodsFor: 'actions' stamp: 'dkh 10/4/2009 10:16'!
applyAdd: addBlock copy: copyBlock merge: mergeBlock remove: removeBlock

	mergeBlock value: self! !

!MetacelloMergeMemberSpec methodsFor: 'actions' stamp: 'dkh 6/5/2009 16:06'!
applyToList: aListSpec

	aListSpec merge: self! !

!MetacelloMergeMemberSpec methodsFor: 'accessing' stamp: 'dkh 10/9/2009 11:43'!
methodUpdateSelector

	^#with:! !

MetacelloMemberSpec subclass: #MetacelloRemoveMemberSpec
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Members'!

!MetacelloRemoveMemberSpec methodsFor: 'actions' stamp: 'dkh 10/4/2009 10:16'!
applyAdd: addBlock copy: copyBlock merge: mergeBlock remove: removeBlock

	removeBlock value: self! !

!MetacelloRemoveMemberSpec methodsFor: 'actions' stamp: 'dkh 06/02/2009 18:27'!
applyToList: aListSpec

	aListSpec remove: self! !

!MetacelloRemoveMemberSpec methodsFor: 'accessing' stamp: 'dkh 6/15/2009 10:27'!
methodUpdateSelector

	^#remove:! !

MetacelloSpec subclass: #MetacelloProjectSpec
	instanceVariableNames: 'name className versionString operator loads preLoadDoIt postLoadDoIt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Specs'!

!MetacelloProjectSpec methodsFor: 'adding' stamp: 'dkh 10/4/2009 11:51'!
addToMetacelloPackages: aMetacelloPackagesSpec

	| spec |
	spec := (aMetacelloPackagesSpec project projectReferenceSpec)
			name: self name;
			projectReference: self copy;
			yourself.
	aMetacelloPackagesSpec addMember: 
		(aMetacelloPackagesSpec addMember 
			name: spec name;
			spec: spec;
			yourself)! !

!MetacelloProjectSpec methodsFor: 'querying' stamp: 'dkh 06/02/2009 20:27'!
className

	^className! !

!MetacelloProjectSpec methodsFor: 'accessing' stamp: 'dkh 06/02/2009 20:31'!
className: aString

	className := aString! !

!MetacelloProjectSpec methodsFor: 'printing' stamp: 'DaleHenrichs 6/14/2010 15:18'!
configMethodBodyOn: aStream indent: indent

	| hasVersionString hasOperator hasProjectPackage hasLoads hasClassName hasPreLoadDoIt hasPostLoadDoIt |
	hasClassName := self className ~~ nil.
	hasVersionString := self versionString ~~ nil.
	hasOperator := operator ~~ nil.
	hasProjectPackage := self projectPackage ~~ nil and: [self projectPackage name ~~ nil or: [ self projectPackage packageRepository ~~ nil ]].
	hasLoads := self loads ~~ nil.
	hasPreLoadDoIt := self preLoadDoIt ~~ nil.
	hasPostLoadDoIt := self postLoadDoIt ~~ nil.
	hasClassName
		ifTrue: [
			hasVersionString | hasOperator | hasProjectPackage | hasLoads
				ifTrue: [ aStream cr; tab: indent + 1 ]
				ifFalse: [ aStream space ].
			aStream nextPutAll: 'className: ', self className printString.
			hasVersionString | hasPreLoadDoIt | hasPostLoadDoIt | hasOperator | hasProjectPackage | hasLoads
				ifTrue: [ aStream nextPut: $; ]].
	hasVersionString
		ifTrue: [ 
			hasClassName | hasOperator | hasProjectPackage | hasLoads | hasPreLoadDoIt | hasPostLoadDoIt
				ifTrue: [ aStream cr; tab: indent + 1 ]
				ifFalse: [ aStream space ].
			aStream nextPutAll: 'versionString: ', self versionString printString.
			 hasPreLoadDoIt | hasPostLoadDoIt | hasOperator | hasProjectPackage | hasLoads
				ifTrue: [ aStream nextPut: $; ]].
	hasPreLoadDoIt
		ifTrue: [ 
			hasClassName | hasOperator | hasProjectPackage | hasLoads  | hasPreLoadDoIt
				ifTrue: [ aStream cr; tab: indent + 1 ]
				ifFalse: [ aStream space ].
			aStream nextPutAll: 'preLoadDoIt: ', self preLoadDoIt printString.
			hasPostLoadDoIt | hasOperator | hasProjectPackage | hasLoads
				ifTrue: [ aStream nextPut: $; ]].
	hasPostLoadDoIt
		ifTrue: [ 
			hasClassName | hasOperator | hasProjectPackage | hasLoads | hasPostLoadDoIt
				ifTrue: [ aStream cr; tab: indent + 1 ]
				ifFalse: [ aStream space ].
			aStream nextPutAll: 'postLoadDoIt: ', self postLoadDoIt printString.
			hasOperator | hasProjectPackage | hasLoads
				ifTrue: [ aStream nextPut: $; ]].
	hasOperator
		ifTrue: [ 
			hasClassName | hasVersionString | hasProjectPackage | hasLoads | hasPreLoadDoIt | hasPostLoadDoIt
				ifTrue: [ aStream cr; tab: indent + 1 ]
				ifFalse: [ aStream space ].
			aStream nextPutAll: 'operator: ', self operator printString.
			hasProjectPackage | hasLoads
				ifTrue: [ aStream nextPut: $; ]].
	hasLoads
		ifTrue: [ 
			hasClassName | hasVersionString | hasOperator | hasProjectPackage | hasPreLoadDoIt | hasPostLoadDoIt
				ifTrue: [ aStream cr; tab: indent + 1 ]
				ifFalse: [ aStream space ].
			aStream nextPutAll: 'loads: #('.
			self loads do: [:str | aStream nextPutAll: str printString, ' ' ].
			aStream nextPut: $).
			hasProjectPackage
				ifTrue: [ aStream nextPut: $; ]].
	hasProjectPackage
		ifTrue: [ | hasName hasRepo |
			hasClassName | hasVersionString | hasOperator | hasLoads | hasPreLoadDoIt | hasPostLoadDoIt
				ifTrue: [ aStream cr; tab: indent + 1 ]
				ifFalse: [ aStream space ].
			hasName := self projectPackage name ~~ nil.
			hasRepo := self projectPackage packageRepository ~~ nil.
			hasName
				ifTrue: [
					aStream nextPutAll: 'file: ', self projectPackage name printString.
					hasRepo ifTrue: [ aStream nextPut: $; ]].
			hasRepo
				ifTrue: [  | repos |
					repos := self projectPackage repositories.
					repos map values size = 1
						ifTrue: [ 
							hasName 
								ifTrue: [ aStream cr; tab: indent + 1 ].
							self projectPackage packageRepository 
								configMethodCascadeOn: aStream 
								lastCascade: true ]
						ifFalse: [
							hasName ifTrue: [ aStream cr ].
							self projectPackage repositories 
								configMethodCascadeOn: aStream 
								indent: indent]]].! !

!MetacelloProjectSpec methodsFor: 'printing' stamp: 'dkh 10/24/2009 19:56'!
configMethodOn: aStream indent: indent

	aStream 
		tab: indent;
		nextPutAll: 'spec '; cr;
		tab: indent + 1;
		nextPutAll: 'name: ', self name printString, ';'.
	self configMethodBodyOn: aStream indent: indent.
	aStream nextPut: $.! !

!MetacelloProjectSpec methodsFor: 'printing' stamp: 'DaleHenrichs 1/21/2010 18:22'!
configShortCutMethodOn: aStream member: aMember indent: indent

	| hasVersionString hasOperator hasProjectPackage hasLoads hasClassName hasPreLoadDoIt hasPostLoadDoIt |
	hasClassName := self className ~~ nil.
	hasVersionString := self versionString ~~ nil.
	hasOperator := operator ~~ nil.
	hasProjectPackage := self projectPackage ~~ nil and: [self projectPackage name ~~ nil or: [ self projectPackage packageRepository ~~ nil ]].
	hasLoads := self loads ~~ nil.
	hasPreLoadDoIt := self preLoadDoIt ~~ nil.
	hasPostLoadDoIt := self postLoadDoIt ~~ nil.
	hasClassName | hasOperator | hasProjectPackage | hasLoads | hasPreLoadDoIt | hasPostLoadDoIt
		ifTrue: [ 
			(aMember methodUpdateSelector == #copy: or: [ aMember methodUpdateSelector == #with: ])
				ifTrue: [ aStream nextPutAll: 'with: ['; cr ]
				ifFalse: [ aStream nextPutAll: 'overrides: ['; cr ].
			aStream 
				tab: indent; nextPutAll: 'spec'.
			self configMethodBodyOn: aStream indent: indent.
			aStream nextPutAll: ' ]'.
			^self ].
	hasVersionString
		ifTrue: [ aStream nextPutAll: 'with: ', self versionString printString ]! !

!MetacelloProjectSpec methodsFor: 'printing' stamp: 'DaleHenrichs 3/9/2010 16:33'!
label

	^self name! !

!MetacelloProjectSpec methodsFor: 'loading' stamp: 'dkh 6/9/2009 10:16'!
load

	self subclassResponsibility! !

!MetacelloProjectSpec methodsFor: 'querying' stamp: 'dkh 10/4/2009 12:04'!
loads
	^ loads! !

!MetacelloProjectSpec methodsFor: 'accessing' stamp: 'dkh 10/4/2009 12:05'!
loads: aCollection

	aCollection setLoadsInMetacelloProject: self! !

!MetacelloProjectSpec methodsFor: 'merging' stamp: 'DaleHenrichs 1/21/2010 19:34'!
mergeMap

	| map |
	map := super mergeMap.
	map at: #name put: name.
	map at: #className put: className.
	map at: #versionString put: versionString.
	map at: #operator put: operator.
	map at: #loads put: loads.
	map at: #preLoadDoIt put: preLoadDoIt.
	map at: #postLoadDoIt put: postLoadDoIt.
	^map! !

!MetacelloProjectSpec methodsFor: 'querying' stamp: 'dkh 06/02/2009 20:32'!
name

	^name! !

!MetacelloProjectSpec methodsFor: 'accessing' stamp: 'dkh 06/02/2009 20:32'!
name: aString

	name := aString! !

!MetacelloProjectSpec methodsFor: 'querying' stamp: 'dkh 10/6/2009 15:39'!
operator

	operator == nil ifTrue: [ ^#>= ].
	^ operator! !

!MetacelloProjectSpec methodsFor: 'accessing' stamp: 'dkh 6/14/2009 13:37'!
operator: anObject
	" #= #~= #> #< #>= #<= #~> "
	
	operator := anObject! !

!MetacelloProjectSpec methodsFor: 'querying' stamp: 'DaleHenrichs 1/21/2010 18:15'!
postLoadDoIt

	^postLoadDoIt! !

!MetacelloProjectSpec methodsFor: 'accessing' stamp: 'DaleHenrichs 1/21/2010 18:15'!
postLoadDoIt: aSymbol

	postLoadDoIt := aSymbol! !

!MetacelloProjectSpec methodsFor: 'querying' stamp: 'DaleHenrichs 1/21/2010 18:16'!
preLoadDoIt

	^preLoadDoIt! !

!MetacelloProjectSpec methodsFor: 'accessing' stamp: 'DaleHenrichs 1/21/2010 18:16'!
preLoadDoIt: aSymbol

	preLoadDoIt := aSymbol! !

!MetacelloProjectSpec methodsFor: 'querying' stamp: 'dkh 10/5/2009 11:47'!
projectPackage

	^nil! !

!MetacelloProjectSpec methodsFor: 'private' stamp: 'dkh 10/4/2009 12:04'!
setLoads: aCollection

	loads := aCollection! !

!MetacelloProjectSpec methodsFor: 'querying' stamp: 'dkh 6/7/2009 09:20'!
versionString
	^ versionString! !

!MetacelloProjectSpec methodsFor: 'accessing' stamp: 'dkh 6/7/2009 09:20'!
versionString: anObject
	versionString := anObject! !

!MetacelloSpec class methodsFor: 'instance creation' stamp: 'dkh 05/01/2009 12:22'!
for: aVersionMap

	^(self platformClass new)
		for: aVersionMap;
		yourself! !

!MetacelloSpec class methodsFor: 'accessing' stamp: 'dkh 04/22/2009 12:22'!
platformClass

	^self! !

!MetacelloSpec methodsFor: 'spec creation' stamp: 'dkh 06/02/2009 16:54'!
addMember

	^MetacelloAddMemberSpec for: self project! !

!MetacelloSpec methodsFor: 'querying' stamp: 'DaleHenrichs 4/9/2010 12:47'!
answers

	^#()! !

!MetacelloSpec methodsFor: 'printing' stamp: 'dkh 10/5/2009 09:14'!
configMethodOn: aStream

	self configMethodOn: aStream indent: 0! !

!MetacelloSpec methodsFor: 'printing' stamp: 'dkh 10/5/2009 09:14'!
configMethodOn: aStream indent: indent

	self subclassResponsibility! !

!MetacelloSpec methodsFor: 'spec creation' stamp: 'dkh 10/4/2009 10:44'!
copyMember

	^MetacelloCopyMemberSpec for: self project! !

!MetacelloSpec methodsFor: 'doits' stamp: 'dkh 12/30/2009 13:17'!
doItBlock: selector

	selector == nil ifTrue: [ ^nil ].
	selector numArgs = 0
		ifTrue: [ ^[ self project configuration perform: selector ] ].
	selector numArgs = 1
		ifTrue: [ ^[:aLoader | self project configuration perform: selector with: aLoader ] ].
	selector numArgs = 2
		ifTrue: [ ^[:aLoader :pkgSpec | self project configuration perform: selector with: aLoader with: pkgSpec ] ].
	^nil! !

!MetacelloSpec methodsFor: 'initialization' stamp: 'dkh 06/02/2009 18:05'!
for: aProject

	project := aProject! !

!MetacelloSpec methodsFor: 'printing' stamp: 'DaleHenrichs 3/9/2010 16:33'!
label

	^self printString! !

!MetacelloSpec methodsFor: 'accessing' stamp: 'dkh 6/18/2009 10:30'!
loader

	loader == nil 
		ifTrue: [ | prjctLoader |
			(prjctLoader := self project loader) == nil
				ifTrue: [ self loader: (self project loaderClass on: self) ]
				ifFalse: [ self loader: prjctLoader ]].
	^loader! !

!MetacelloSpec methodsFor: 'accessing' stamp: 'dkh 6/18/2009 09:46'!
loader: aLoader

	"We're interested in propogating the loader state, _except_ for the spec"
	loader := aLoader copy.
	loader spec: self! !

!MetacelloSpec methodsFor: 'merging' stamp: 'dkh 06/04/2009 14:40'!
mergeMap

	^Dictionary new.! !

!MetacelloSpec methodsFor: 'spec creation' stamp: 'dkh 06/02/2009 16:54'!
mergeMember

	^MetacelloMergeMemberSpec for: self project! !

!MetacelloSpec methodsFor: 'merging' stamp: 'dkh 9/23/2009 11:40'!
mergeSpec: aSpec

	| newSpec nonOverridable |
	newSpec := self copy.
	nonOverridable := self nonOverridable.
	aSpec mergeMap keysAndValuesDo: [:key :value |
		(nonOverridable includes: key)
			ifFalse: [ value ~~ nil ifTrue: [ newSpec instVarNamed: key asString put: value ]]].
	^newSpec! !

!MetacelloSpec methodsFor: 'merging' stamp: 'dkh 9/23/2009 11:41'!
nonOverridable

	^#()! !

!MetacelloSpec methodsFor: 'copying' stamp: 'dkh 10/4/2009 18:56'!
postCopy

	super postCopy.
	loader ~~ nil  ifTrue: [ self loader: loader ]! !

!MetacelloSpec methodsFor: 'doits' stamp: 'dkh 12/30/2009 13:20'!
postLoadDoIt
	"noop unless non-nil value returned"

	^nil! !

!MetacelloSpec methodsFor: 'doits' stamp: 'dkh 12/30/2009 13:17'!
postLoadDoItBlock

	^self doItBlock: self postLoadDoIt! !

!MetacelloSpec methodsFor: 'doits' stamp: 'dkh 12/30/2009 13:20'!
preLoadDoIt
	"noop unless non-nil value returned"
	
	^nil! !

!MetacelloSpec methodsFor: 'doits' stamp: 'dkh 12/30/2009 13:17'!
preLoadDoItBlock

	^self doItBlock: self preLoadDoIt! !

!MetacelloSpec methodsFor: 'printing' stamp: 'dkh 10/5/2009 11:08'!
printOn: aStream

	self configMethodOn: aStream indent: 0! !

!MetacelloSpec methodsFor: 'querying' stamp: 'dkh 06/02/2009 14:42'!
project

	^project! !

!MetacelloSpec methodsFor: 'spec creation' stamp: 'dkh 06/02/2009 16:54'!
removeMember

	^MetacelloRemoveMemberSpec for: self project! !

MetacelloSpec subclass: #MetacelloValueHolderSpec
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Specs'!

!MetacelloValueHolderSpec methodsFor: 'printing' stamp: 'dkh 10/5/2009 12:23'!
configMethodOn: aStream indent: indent

	aStream tab: indent; nextPutAll: 'spec value: ', self value printString! !

!MetacelloValueHolderSpec methodsFor: 'merging' stamp: 'dkh 6/5/2009 11:46'!
mergeMap

	| map |
	map := super mergeMap.
	map at: #value put: value.
	^map! !

!MetacelloValueHolderSpec methodsFor: 'private' stamp: 'dkh 9/13/2009 07:46'!
setAuthorInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setAuthor: self! !

!MetacelloValueHolderSpec methodsFor: 'private' stamp: 'dkh 6/5/2009 10:49'!
setBlessingInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setBlessing: self! !

!MetacelloValueHolderSpec methodsFor: 'private' stamp: 'dkh 6/5/2009 11:54'!
setDescriptionInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setDescription: self! !

!MetacelloValueHolderSpec methodsFor: 'private' stamp: 'dkh 9/13/2009 07:47'!
setTimestampInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setTimestamp: self! !

!MetacelloValueHolderSpec methodsFor: 'querying' stamp: 'dkh 6/5/2009 11:46'!
value
	^ value! !

!MetacelloValueHolderSpec methodsFor: 'accessing' stamp: 'dkh 6/5/2009 11:46'!
value: anObject
	value := anObject! !

MetacelloSpec subclass: #MetacelloVersionSpec
	instanceVariableNames: 'versionString blessing description author timestamp preLoadDoIt postLoadDoIt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Specs'!

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 9/13/2009 07:42'!
author

	author == nil 
		ifTrue: [
			^self project valueHolderSpec
				value: '';
				yourself].
	^ author! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 9/13/2009 07:43'!
author: anObject

	anObject setAuthorInMetacelloVersion: self! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 8/26/2009 10:10'!
blessing

	blessing == nil 
		ifTrue: [
			^self project valueHolderSpec
				value: self project defaultBlessing;
				yourself].
	^ blessing! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 6/5/2009 10:44'!
blessing: anObject

	anObject setBlessingInMetacelloVersion: self! !

!MetacelloVersionSpec methodsFor: 'printing' stamp: 'dkh 10/5/2009 13:22'!
configMethodOn: aStream for: spec selector: selector indent: indent

	spec == nil ifTrue: [ ^self ].
	aStream tab: indent; nextPutAll: 'spec ', selector, ' ['; cr.
	spec configMethodOn: aStream indent: indent + 1.
	aStream nextPutAll: ' ].'; cr! !

!MetacelloVersionSpec methodsFor: 'printing' stamp: 'dkh 12/30/2009 13:26'!
configMethodOn: aStream indent: indent

	self configMethodValueOn: aStream for: self getBlessing selector: 'blessing:' indent: indent.
	self configMethodValueOn: aStream for: self getDescription selector: 'description:' indent: indent.
	self configMethodValueOn: aStream for: self getPreLoadDoIt selector: 'preLoadDoIt:' indent: indent.
	self configMethodValueOn: aStream for: self getPostLoadDoIt selector: 'postLoadDoIt:' indent: indent.
	self configMethodValueOn: aStream for: self getAuthor selector: 'author:' indent: indent.
	self configMethodValueOn: aStream for: self getTimestamp selector: 'timestamp:' indent: indent.! !

!MetacelloVersionSpec methodsFor: 'printing' stamp: 'dkh 10/7/2009 01:15'!
configMethodValueOn: aStream for: spec selector: selector indent: indent

	spec == nil ifTrue: [ ^self ].
	aStream tab: indent; nextPutAll: 'spec ', selector, ' ', spec value printString, '.'; cr! !

!MetacelloVersionSpec methodsFor: 'printing' stamp: 'DaleHenrichs 3/23/2010 19:08'!
configSpawnMethodOn: aStream indent: indent

	self configMethodValueOn: aStream for: self getAuthor selector: 'author:' indent: indent.
	self configMethodValueOn: aStream for: self getTimestamp selector: 'timestamp:' indent: indent.
	self configMethodValueOn: aStream for: self getBlessing selector: 'blessing:' indent: indent.! !

!MetacelloVersionSpec methodsFor: 'private' stamp: 'dkh 10/7/2009 14:45'!
createVersion

	^self versionClass fromSpec: self! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 8/26/2009 10:35'!
description

	description == nil 
		ifTrue: [
			^self project valueHolderSpec
				value: '';
				yourself].
	^ description! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 6/5/2009 11:40'!
description: anObject

	anObject setDescriptionInMetacelloVersion: self! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 10/3/2009 16:42'!
getAuthor
	^author! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 10/3/2009 16:41'!
getBlessing
	^blessing! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 10/3/2009 16:41'!
getDescription
	^description! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 12/30/2009 13:25'!
getPostLoadDoIt
	^postLoadDoIt! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 12/30/2009 13:25'!
getPreLoadDoIt
	^preLoadDoIt! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 10/3/2009 16:42'!
getTimestamp
	^timestamp! !

!MetacelloVersionSpec methodsFor: 'testing' stamp: 'dkh 12/14/2009 10:20'!
isPartiallyCurrent: notLoadedMatters useEquality: useEquality

	self subclassResponsibility! !

!MetacelloVersionSpec methodsFor: 'testing' stamp: 'dkh 12/14/2009 10:20'!
isPossibleBaseline

	self subclassResponsibility! !

!MetacelloVersionSpec methodsFor: 'printing' stamp: 'DaleHenrichs 3/9/2010 16:34'!
label

	^self versionString, ' [', self projectLabel, ']'! !

!MetacelloVersionSpec methodsFor: 'merging' stamp: 'dkh 12/30/2009 14:17'!
mergeMap

	| map |
	map := super mergeMap.
	map at: #versionString put: versionString.
	map at: #blessing put: blessing.
	map at: #description put: description.
	map at: #author put: author.
	map at: #timestamp put: timestamp.
	map at: #preLoadDoIt put: preLoadDoIt.
	map at: #postLoadDoIt put: postLoadDoIt.
	^map! !

!MetacelloVersionSpec methodsFor: 'copying' stamp: 'dkh 10/4/2009 19:10'!
postCopy

	super postCopy.
	blessing := blessing copy.
	description := description copy.
	author := author copy.
	timestamp := timestamp copy.! !

!MetacelloVersionSpec methodsFor: 'querying' stamp: 'dkh 12/30/2009 13:14'!
postLoadDoIt

	^postLoadDoIt! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 12/30/2009 13:15'!
postLoadDoIt: aSymbol

	postLoadDoIt := aSymbol! !

!MetacelloVersionSpec methodsFor: 'querying' stamp: 'dkh 12/30/2009 13:15'!
preLoadDoIt

	^preLoadDoIt! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 12/30/2009 13:15'!
preLoadDoIt: aSymbol

	preLoadDoIt := aSymbol! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'DaleHenrichs 3/9/2010 16:31'!
projectLabel

	^self project label! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 9/13/2009 07:44'!
setAuthor: anObject
	author := anObject! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 6/5/2009 10:44'!
setBlessing: anObject
	blessing := anObject! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 6/5/2009 11:40'!
setDescription: anObject
	description := anObject! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 9/13/2009 07:44'!
setTimestamp: anObject
	timestamp := anObject! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 9/13/2009 07:42'!
timestamp

	timestamp == nil 
		ifTrue: [
			^self project valueHolderSpec
				value: '';
				yourself].
	^ timestamp! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 9/13/2009 07:44'!
timestamp: anObject

	anObject setTimestampInMetacelloVersion: self! !

!MetacelloVersionSpec methodsFor: 'private' stamp: 'dkh 10/7/2009 14:44'!
versionClass

	^MetacelloVersion! !

!MetacelloVersionSpec methodsFor: 'querying' stamp: 'dkh 8/31/2009 05:10'!
versionString
	versionString == nil ifTrue: [ ^'' ].
	^ versionString! !

!MetacelloVersionSpec methodsFor: 'accessing' stamp: 'dkh 6/4/2009 18:50'!
versionString: anObject
	versionString := anObject! !

Object subclass: #MetacelloSpecLoader
	instanceVariableNames: 'spec'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Loaders'!

!MetacelloSpecLoader class methodsFor: 'instance creation' stamp: 'dkh 04/23/2009 11:24'!
on: aMetacelloPackagesSpec

	^(self new) 
		spec: aMetacelloPackagesSpec;
		yourself! !

!MetacelloSpecLoader methodsFor: 'actions' stamp: 'dkh 04/23/2009 11:21'!
load

	self subclassResponsibility! !

!MetacelloSpecLoader methodsFor: 'accessing' stamp: 'dkh 6/12/2009 14:17'!
loadType
	"#atomic or #linear"
	
	^self project loadType! !

!MetacelloSpecLoader methodsFor: 'accessing' stamp: 'dkh 6/12/2009 14:17'!
project

	^self spec project! !

!MetacelloSpecLoader methodsFor: 'accessing' stamp: 'dkh 04/23/2009 11:20'!
spec

	^spec! !

!MetacelloSpecLoader methodsFor: 'accessing' stamp: 'dkh 04/23/2009 11:20'!
spec: aMetacelloPackagesSpec

	spec := aMetacelloPackagesSpec! !

!MetacelloSpecLoader methodsFor: 'actions' stamp: 'dkh 04/23/2009 11:48'!
unload

	self subclassResponsibility! !

Object subclass: #MetacelloVersionConstructor
	instanceVariableNames: 'root configuration project attributeMap attributeOrder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Model'!

!MetacelloVersionConstructor class methodsFor: 'instance creation' stamp: 'dkh 10/3/2009 17:27'!
on: aConfig

	^(self new)
		on: aConfig;
		yourself! !

!MetacelloVersionConstructor class methodsFor: 'instance creation' stamp: 'dkh 10/14/2009 16:52'!
on: aConfig project: aProject

	^(self new)
		project: aProject;
		on: aConfig;
		yourself! !

!MetacelloVersionConstructor class methodsFor: 'instance creation' stamp: 'DaleHenrichs 3/12/2010 00:13'!
spawnPackageMethodIn: aConfig category: methodCategory named: newSelector sourceVersion: sourceVersionString targetVersion: targetVersionString blessing: blessing

	^self new
		spawnPackageMethodIn: aConfig 
		category: methodCategory 
		named: newSelector
		sourceVersion: sourceVersionString 
		targetVersion: targetVersionString
		blessing: blessing! !

!MetacelloVersionConstructor class methodsFor: 'instance creation' stamp: 'DaleHenrichs 3/23/2010 19:14'!
spawnPackageMethodIn: aConfig named: newSelector sourceVersion: sourceVersionString targetVersion: targetVersionString

	^self 
		spawnPackageMethodIn: aConfig 
		category: 'versions'
		named: newSelector 
		sourceVersion: sourceVersionString 
		targetVersion: targetVersionString 
		blessing: #development! !

!MetacelloVersionConstructor class methodsFor: 'instance creation' stamp: 'DaleHenrichs 3/12/2010 00:15'!
spawnPackageMethodIn: aConfig named: newSelector sourceVersion: sourceVersionString targetVersion: targetVersionString blessing: blessing

	^self new
		spawnPackageMethodIn: aConfig 
		category: 'versions'
		named: newSelector 
		sourceVersion: sourceVersionString 
		targetVersion: targetVersionString
		blessing: blessing! !

!MetacelloVersionConstructor class methodsFor: 'instance creation' stamp: 'dkh 10/7/2009 13:57'!
updatePackageMethodIn: aConfig sourceVersion: sourceVersionString

	^self new
		updatePackageMethodIn: aConfig 
		sourceVersion: sourceVersionString! !

!MetacelloVersionConstructor methodsFor: 'accessing' stamp: 'dkh 10/3/2009 16:28'!
attributeMap

	attributeMap == nil ifTrue: [ attributeMap := Dictionary new ].
	^attributeMap! !

!MetacelloVersionConstructor methodsFor: 'accessing' stamp: 'dkh 10/5/2009 16:41'!
attributeOrder

	attributeOrder == nil ifTrue: [ attributeOrder := OrderedCollection new ].
	^attributeOrder! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/7/2009 00:13'!
author: aBlockOrString

	aBlockOrString setAuthorInMetacelloConfig: self! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/7/2009 00:12'!
blessing: aBlockOrString

	aBlockOrString setBlessingInMetacelloConfig: self! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/3/2009 17:03'!
className: anObject

	self root className: anObject! !

!MetacelloVersionConstructor methodsFor: 'private' stamp: 'dkh 1/7/2010 18:57'!
collectAllVersionsFromVersionImportPragmasInto: versionMap using: executionBlock satisfiedPragmas: pragmaDict 
	| defined done completed |
	done := false.
	completed := IdentitySet new.
	[done]
		whileFalse: [done := true.
			pragmaDict
				keysAndValuesDo: [:versionString :pragmaColl | 
					| versionSpec |
					versionSpec := nil.
					defined := false.
					pragmaColl
						do: [:pragma | (completed includes: pragma)
								ifFalse: [done := false.
									(pragma argumentAt: 2)
										do: [:importedVersion | | version | (version := versionMap
															at: importedVersion
															ifAbsent: []) ~~ nil
												ifTrue: [defined := true.
													versionSpec == nil
														ifTrue: [versionSpec := version spec copy]
														ifFalse: [versionSpec := versionSpec mergeSpec: version spec copy].
													versionSpec versionString: versionString.
													executionBlock value: versionSpec value: pragma]].
									defined
										ifTrue: [completed add: pragma]]].
					defined
						ifTrue: [ | version importedVersions |
							importedVersions := OrderedCollection new.
							version := versionSpec createVersion.
							pragmaColl do: [:pragma | importedVersions addAll: (pragma argumentAt: 2) ].
							version importedVersions: importedVersions.
							versionMap at: versionSpec versionString put: version ].
					self reset]]! !

!MetacelloVersionConstructor methodsFor: 'private' stamp: 'djr 12/29/2009 18:01'!
collectAllVersionsFromVersionPragmasInto: versionMap using: executionBlock 
	| defined versionPragmaDict |
	versionPragmaDict := self extractVersionPragmas.
	versionPragmaDict
		keysAndValuesDo: [:versionString :pragmaColl | 
			| versionSpec |
			versionSpec := self project versionSpec.
			versionSpec versionString: versionString.
			defined := false.
			pragmaColl
				do: [:pragma | 
					defined := true.
					executionBlock value: versionSpec value: pragma].
			defined
				ifTrue: [versionMap at: versionSpec versionString put: versionSpec createVersion].
			self reset]! !

!MetacelloVersionConstructor methodsFor: 'accessing' stamp: 'dkh 10/3/2009 16:26'!
configuration

	^configuration! !

!MetacelloVersionConstructor methodsFor: 'accessing' stamp: 'dkh 10/3/2009 16:26'!
configuration: aConfig

	configuration := aConfig! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/7/2009 00:13'!
description: aBlockOrString

	aBlockOrString setDescriptionInMetacelloConfig: self! !

!MetacelloVersionConstructor methodsFor: 'private' stamp: 'dkh 10/7/2009 00:53'!
extractAllPragmas

	| aDict |
	aDict := Dictionary new.
	self extractPragmas: #version: into: aDict.
	self extractPragmas: #version:imports: into: aDict.
	^aDict! !

!MetacelloVersionConstructor methodsFor: 'private' stamp: 'DaleHenrichs 01/15/2010 14:18'!
extractPragmas: pragmaKeyword into: versionDict

	| versionString  pragmas |
	(Pragma 
		allNamed: pragmaKeyword
		in: self configuration class) do: [:pragma |
			versionString := pragma argumentAt: 1.
			pragmas := versionDict 
				at: versionString 
				ifAbsent: [ | list |
					list := OrderedCollection new.
					versionDict at: versionString put: list.
					list ].
			pragmas add: pragma ].! !

!MetacelloVersionConstructor methodsFor: 'private' stamp: 'dkh 10/7/2009 00:52'!
extractVersionImportPragmas

	| aDict |
	aDict := Dictionary new.
	self extractPragmas: #version:imports: into: aDict.
	^aDict! !

!MetacelloVersionConstructor methodsFor: 'private' stamp: 'dkh 10/7/2009 00:53'!
extractVersionPragmas

	| aDict |
	aDict := Dictionary new.
	self extractPragmas: #version: into: aDict.
	^aDict! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/3/2009 17:24'!
file: anObject

	self root file: anObject! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/5/2009 16:42'!
for: attribute do: aBlock

	self attributeMap at: attribute put: aBlock.
	self attributeOrder add: attribute! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/9/2009 11:30'!
group: aString overrides: aStringOrCollection

	| spec |
	spec := 
		(self project groupSpec)
			name: aString;
			includes: aStringOrCollection;
			yourself.
	self root packages add: spec.! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/9/2009 11:30'!
group: aString with: aStringOrCollection

	| spec |
	spec := 
		(self project groupSpec)
			name: aString;
			includes: aStringOrCollection;
			yourself.
	self root packages merge: spec.! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/3/2009 17:11'!
includes: anObject

	self root includes: anObject! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/4/2009 11:11'!
loads: anObject

	self root loads: anObject! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/3/2009 17:04'!
name: anObject

	self root name: anObject! !

!MetacelloVersionConstructor methodsFor: 'initialization' stamp: 'djr 12/29/2009 18:01'!
on: aConfig 
	| versionMap executionBlock |
	self configuration: aConfig.
	versionMap := Dictionary new.
	executionBlock := self specResolverBlock.
	self collectAllVersionsFromVersionPragmasInto: versionMap using: executionBlock.
	self
		collectAllVersionsFromVersionImportPragmasInto: versionMap
		using: executionBlock
		satisfiedPragmas: (self versionImportPragmasVerifiedDefinedIn: versionMap).
	self project map: versionMap.
	self project configuration: aConfig! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/8/2009 20:58'!
operator: anObject

	self root operator: anObject! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/7/2009 16:39'!
package: aString

	| spec |
	spec := 
		(self project packageSpec)
			name: aString;
			yourself.
	self root packages add: spec.! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/9/2009 11:22'!
package: aString overrides: aBlock

	| spec |
	spec := 
		(self project packageSpec)
			name: aString;
			yourself.
	self root packages add: spec.
	self with: spec during: aBlock! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/9/2009 11:22'!
package: aString with: aBlockOrString

	aBlockOrString setPackage: aString withInMetacelloConfig: self! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 12/30/2009 13:08'!
postLoadDoIt: aSymbol

	self validateDoItSelector: aSymbol.
	self root postLoadDoIt: aSymbol! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 12/30/2009 13:08'!
preLoadDoIt: aSymbol

	self validateDoItSelector: aSymbol.
	self root preLoadDoIt: aSymbol! !

!MetacelloVersionConstructor methodsFor: 'private' stamp: 'DaleHenrichs 03/12/2010 01:52'!
prepareForMethodUpdate: aConfig sourceVersion: sourceVersionString generating: generateBlock

	| pragmaDict versionSpecs pragmaMap updatedPackageSpecs updatedPackageSpecsMap reversed pragmaColl seenUpdatedPackageSpecs |
	self configuration: aConfig.
	versionSpecs := Dictionary new.
	pragmaMap := Dictionary new.
	pragmaDict := self extractAllPragmas.
	pragmaColl := pragmaDict at: sourceVersionString ifAbsent: [ ^self ].
	pragmaColl do: [:pragma | | specs versionSpec |
		specs := Dictionary new.
		self configuration perform: pragma selector with: self.
		self attributeMap keysAndValuesDo: [:attribute :block | 
			versionSpec := self project versionSpec.
			versionSpec versionString: sourceVersionString.
			specs at: attribute put: versionSpec.
			self 
				with: versionSpec
				during: block ].
		versionSpecs at: pragma selector put: { specs. self attributeOrder. nil. }.
		pragmaMap at: pragma selector put: pragma.
		self reset ].
	versionSpecs keysAndValuesDo: [:selector :ar |
		updatedPackageSpecsMap := Dictionary new.
		seenUpdatedPackageSpecs := Dictionary new.
		ar at: 3 put: updatedPackageSpecsMap.
		reversed := aConfig project attributes reverse.
		reversed do: [:attribute | | vs |
			(vs := (ar at: 1) at: attribute ifAbsent: []) ~~ nil
				ifTrue: [ 
					updatedPackageSpecs := vs updatedPackageSpecs.
					updatedPackageSpecs associations do: [:assoc | | filename |
						(filename := seenUpdatedPackageSpecs at: assoc key ifAbsent:[]) == nil
							ifTrue: [ 
								assoc value == #uptodate
									ifTrue: [ 
										"#uptodate means that the spec is up-to-date and we mark it as
										 seen so that the spec is not update for a 'later' spec"
										seenUpdatedPackageSpecs at: assoc key put: #uptodate ]
									ifFalse: [ seenUpdatedPackageSpecs at: assoc key put: assoc value file ]]
							ifFalse: [
								"if the spec was already seen as up-to-date or the file is the same as 
								 the one already seen don't propogate the file"
								(filename == #uptodate or: [ assoc value == #uptodate or: [ filename = assoc value file ]])
									ifTrue: [ updatedPackageSpecs removeKey: assoc key ]]].
					updatedPackageSpecsMap at: attribute put: updatedPackageSpecs ]].
		"clear out #uptodate markers"
		updatedPackageSpecsMap valuesDo: [:d |
			d associations do: [:assoc |
				assoc value == #uptodate ifTrue: [ d removeKey: assoc key ]]].
		"remove shadowed packages"
		1 to: reversed size do: [:index | | attribute d |
			attribute := reversed at: index.
			((d := updatedPackageSpecsMap at: attribute ifAbsent: []) ~~ nil and: [ d keys size > 0 ])
				ifTrue: [
					index + 1 to: reversed size do: [:shadowIndex |
						d keysDo: [:key | | dict |
							(dict := updatedPackageSpecsMap at: (reversed at: shadowIndex) ifAbsent: []) ~~ nil
								ifTrue: [ dict removeKey: key ifAbsent: []]]]]].
		updatedPackageSpecsMap keys do: [:key | | d |
			d := updatedPackageSpecsMap at: key.
			d isEmpty ifTrue: [ updatedPackageSpecsMap removeKey: key ]].
		"ready to generate source for method"
		generateBlock value:selector value: pragmaMap value: ar value: updatedPackageSpecsMap]! !

!MetacelloVersionConstructor methodsFor: 'accessing' stamp: 'dkh 10/3/2009 17:36'!
project

	project == nil ifTrue: [ project := MetacelloMCProject new].
	^project! !

!MetacelloVersionConstructor methodsFor: 'accessing' stamp: 'dkh 10/14/2009 16:55'!
project: aProject

	project := aProject! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/9/2009 11:38'!
project: aString copyFrom: oldSpecName with: aBlock

	| spec projectSpec |
	projectSpec := 
		(self project projectSpec)
			name: aString;
			projectPackage: self project packageSpec;
			yourself.
	spec := 
		(self project projectReferenceSpec)
			name: aString;
			projectReference: projectSpec;
			yourself.
	self root packages copy: oldSpecName to: spec.
	self with: projectSpec during: aBlock! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/9/2009 11:15'!
project: aString overrides: aBlock

	| spec projectSpec |
	projectSpec := 
		(self project projectSpec)
			name: aString;
			projectPackage: self project packageSpec;
			yourself.
	projectSpec projectPackage: self project packageSpec.
	spec := 
		(self project projectReferenceSpec)
			name: aString;
			projectReference: projectSpec;
			yourself.
	self root packages add: spec.
	self with: projectSpec during: aBlock! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/9/2009 11:16'!
project: aString with: aBlockOrString

	aBlockOrString setProject: aString withInMetacelloConfig: self! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/3/2009 16:54'!
projectPackage: aBlock

	| spec |
	(spec := self root projectPackage) == nil
		ifTrue: [ 
			spec := self project packageSpec.
			self root projectPackage: spec ].
	self with: spec during: aBlock! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/7/2009 00:39'!
removeGroup: aString

	| spec |
	spec := 
		(self project groupSpec)
			name: aString;
			yourself.
	self root packages remove: spec.! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/7/2009 00:39'!
removePackage: aString

	| spec |
	spec := 
		(self project packageSpec)
			name: aString;
			yourself.
	self root packages remove: spec.! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/7/2009 00:40'!
removeProject: aString

	| spec |
	spec := 
		(self project projectReferenceSpec)
			name: aString;
			yourself.
	self root packages remove: spec.! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/3/2009 16:50'!
repositories: aBlock

	self with: self root repositories during: aBlock! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/6/2009 13:36'!
repository: anObject

	self root repository: anObject! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/22/2009 09:02'!
repository: description username: username password: password

	self root repository: description username: username password: password! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/3/2009 17:10'!
requires: anObject

	self root requires: anObject! !

!MetacelloVersionConstructor methodsFor: 'initialization' stamp: 'dkh 10/5/2009 16:42'!
reset

	attributeMap := attributeOrder := nil! !

!MetacelloVersionConstructor methodsFor: 'accessing' stamp: 'dkh 10/3/2009 16:36'!
root

	^root! !

!MetacelloVersionConstructor methodsFor: 'accessing' stamp: 'dkh 10/3/2009 16:37'!
root: aMetacelloSpec

	root := aMetacelloSpec! !

!MetacelloVersionConstructor methodsFor: 'api callbacks' stamp: 'dkh 10/7/2009 00:10'!
setAuthorWithBlock: aBlock

	| spec |
	(spec := self root getAuthor) == nil
		ifTrue: [ 
			spec := self project valueHolderSpec.
			self root setAuthor: spec ].
	self with: spec during: aBlock! !

!MetacelloVersionConstructor methodsFor: 'api callbacks' stamp: 'dkh 10/7/2009 00:10'!
setAuthorWithString: aString

	self root author: aString! !

!MetacelloVersionConstructor methodsFor: 'api callbacks' stamp: 'dkh 10/7/2009 00:10'!
setBlessingWithBlock: aBlock

	| spec |
	(spec := self root getBlessing) == nil
		ifTrue: [ 
			spec := self project valueHolderSpec.
			self root setBlessing: spec ].
	self with: spec during: aBlock! !

!MetacelloVersionConstructor methodsFor: 'api callbacks' stamp: 'dkh 10/7/2009 00:10'!
setBlessingWithString: aString

	self root blessing: aString! !

!MetacelloVersionConstructor methodsFor: 'api callbacks' stamp: 'dkh 10/7/2009 00:10'!
setDescriptionWithBlock: aBlock

	| spec |
	(spec := self root getDescription) == nil
		ifTrue: [ 
			spec := self project valueHolderSpec.
			self root setDescription: spec ].
	self with: spec during: aBlock! !

!MetacelloVersionConstructor methodsFor: 'api callbacks' stamp: 'dkh 10/7/2009 00:11'!
setDescriptionWithString: aString

	self root description: aString! !

!MetacelloVersionConstructor methodsFor: 'api callbacks' stamp: 'dkh 10/9/2009 11:21'!
setPackage: aString withBlock: aBlock

	| spec |
	spec := 
		(self project packageSpec)
			name: aString;
			yourself.
	self root packages merge: spec.
	self with: spec during: aBlock! !

!MetacelloVersionConstructor methodsFor: 'api callbacks' stamp: 'dkh 10/9/2009 11:21'!
setPackage: aString withString: aFile

	| spec |
	spec := 
		(self project packageSpec)
			name: aString;
			file: aFile;
			yourself.
	self root packages merge: spec.! !

!MetacelloVersionConstructor methodsFor: 'api callbacks' stamp: 'dkh 10/9/2009 11:14'!
setProject: aString withBlock: aBlock

	| spec projectSpec |
	projectSpec := 
		(self project projectSpec)
			name: aString;
			projectPackage: self project packageSpec;
			yourself.
	projectSpec projectPackage: self project packageSpec.
	spec := 
		(self project projectReferenceSpec)
			name: aString;
			projectReference: projectSpec;
			yourself.
	self root packages merge: spec.
	self with: projectSpec during: aBlock! !

!MetacelloVersionConstructor methodsFor: 'api callbacks' stamp: 'dkh 10/9/2009 11:14'!
setProject: aString withString: versionString

	| spec projectSpec |
	projectSpec := 
		(self project projectSpec)
			name: aString;
			versionString: versionString;
			yourself.
	spec := 
		(self project projectReferenceSpec)
			name: aString;
			projectReference: projectSpec;
			yourself.
	self root packages merge: spec.! !

!MetacelloVersionConstructor methodsFor: 'api callbacks' stamp: 'dkh 10/7/2009 00:11'!
setTimestampWithBlock: aBlock

	| spec |
	(spec := self root getTimestamp) == nil
		ifTrue: [ 
			spec := self project valueHolderSpec.
			self root setTimestamp: spec ].
	self with: spec during: aBlock! !

!MetacelloVersionConstructor methodsFor: 'api callbacks' stamp: 'dkh 10/7/2009 00:11'!
setTimestampWithString: aString

	self root timestamp: aString! !

!MetacelloVersionConstructor methodsFor: 'initialization' stamp: 'DaleHenrichs 3/12/2010 00:12'!
spawnPackageMethodIn: aConfig category: methodCategory named: newSelector sourceVersion: sourceVersionString targetVersion: targetVersionString blessing: blessing

	self 
		prepareForMethodUpdate: aConfig 
		sourceVersion: sourceVersionString 
		generating: [:selector :pragmaMap :ar :updatedPackageSpecsMap | | strm |
			strm := WriteStream on: String new.
			strm 
				nextPutAll: newSelector asString, ' spec'; cr;
				tab; nextPutAll: 
						'<version: ', 
						targetVersionString printString, 
						' imports: #(', sourceVersionString printString, 
						')>';cr.
			(ar at: 2) do: [:attribute | | vs d |
				vs := (ar at: 1) at: attribute.
				(d := updatedPackageSpecsMap at: attribute ifAbsent: []) ~~ nil
					ifTrue: [ 
						vs packagesSpec list do: [:member | | x |
							x := d at: member spec name ifAbsent: [].
						 	member spec updateForSpawnMethod: x ]]
					ifFalse: [
						vs packagesSpec list do: [:member |
							member spec updateForSpawnMethod: member spec copy ]].
				strm cr; tab; nextPutAll: 'spec for: ', attribute printString, ' do: ['; cr.
				attribute == #common
					ifTrue: [
						vs 
							author: MetacelloPlatform current authorName;
							timestamp: MetacelloPlatform current timestamp.
						blessing ~~ nil
							ifTrue: [ vs blessing: blessing ]].
				vs configSpawnMethodOn: strm indent: 2.
				strm nextPutAll: '].'].
				(aConfig class	
					compile: strm contents
					classified: methodCategory) == nil 
						ifTrue: [ self error: 'Error compiling the method' ]].
	^true! !

!MetacelloVersionConstructor methodsFor: 'private' stamp: 'djr 12/29/2009 18:02'!
specResolverBlock
	^ [:versionSpec :pragma | (pragma argumentAt: 1)
			= versionSpec versionString
		ifTrue: [self configuration perform: pragma selector with: self.
			self project attributes
				do: [:attribute | | block | (block := self attributeMap
									at: attribute
									ifAbsent: []) ~~ nil
						ifTrue: [self with: versionSpec during: block]]]]! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 1/4/2010 18:56'!
supplyingAnswers: aCollection

	self root answers: aCollection! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/7/2009 00:14'!
timestamp: aBlockOrString

	aBlockOrString setTimestampInMetacelloConfig: self! !

!MetacelloVersionConstructor methodsFor: 'initialization' stamp: 'dkh 12/9/2009 10:27'!
updatePackageMethodIn: aConfig sourceVersion: sourceVersionString

	self 
		prepareForMethodUpdate: aConfig 
		sourceVersion: sourceVersionString 
		generating: [:selector :pragmaMap :ar :updatedPackageSpecsMap | | strm pragma |
			updatedPackageSpecsMap isEmpty ifTrue: [ ^false ].
			strm := WriteStream on: String new.
			strm 
				nextPutAll: selector asString, ' spec'; cr;
				tab; nextPutAll: '<version: ', sourceVersionString printString.
			pragma := pragmaMap at: selector.
			pragma numArgs = 2
				ifTrue: [
					strm nextPutAll: ' imports: #('.
					(pragma argumentAt: 2) do: [:versionString |
						strm nextPutAll: versionString printString; space ]].
			strm nextPutAll: ')>';cr.
			(ar at: 2) do: [:attribute | | vs d |
				vs := (ar at: 1) at: attribute.
				(d := updatedPackageSpecsMap at: attribute ifAbsent: []) ~~ nil
					ifTrue: [ 
						vs packagesSpec list do: [:member | 
							member spec file ~~ nil
								ifTrue: [ | x |
									(((x := d at: member spec name ifAbsent: []) ~~ nil) and: [ x ~~ #uptodate ])
						 				ifTrue: [ member spec file: x file ]]]].
				strm cr; tab; nextPutAll: 'spec for: ', attribute printString, ' do: ['; cr.
				attribute == #common
					ifTrue: [ 
						vs 
							author: MetacelloPlatform current authorName;
							timestamp: MetacelloPlatform current timestamp ].
				vs configMethodOn: strm indent: 2.
				strm nextPutAll: '].'].
			(aConfig class	
					compile: strm contents
					classified: (aConfig class whichCategoryIncludesSelector: pragma selector)) == nil
						ifTrue: [ self error: 'Error compiling the method' ]].
	^true! !

!MetacelloVersionConstructor methodsFor: 'validation' stamp: 'dkh 12/30/2009 14:23'!
validateDoItSelector: anObject

	anObject == nil ifTrue: [ ^ self ].
	anObject isSymbol ifFalse: [ self error: 'Invalid message selector for doit: ', anObject printString ].! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/3/2009 16:47'!
value: anObject

	self root value: anObject! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/7/2009 01:11'!
version: anObject

	self root versionString: anObject! !

!MetacelloVersionConstructor methodsFor: 'private' stamp: 'djr 12/29/2009 18:02'!
versionImportPragmasVerifiedDefinedIn: versionMap 
	| pragmaDict |
	pragmaDict := self extractVersionImportPragmas.
	pragmaDict
		keysAndValuesDo: [:versionString :pragmaColl | pragmaColl
				do: [:pragma | (pragma argumentAt: 2)
						do: [:importedVersion | versionMap
								at: importedVersion
								ifAbsent: [pragmaDict
										at: importedVersion
										ifAbsent: [^ self error: 'The imported version:' , importedVersion printString , ' referenced from the method: ' , pragma selector printString , ' in configuration ' , configuration class printString , ' has not been defined.']]]]].
	^ pragmaDict! !

!MetacelloVersionConstructor methodsFor: 'api' stamp: 'dkh 10/3/2009 17:03'!
versionString: anObject

	self root versionString: anObject! !

!MetacelloVersionConstructor methodsFor: 'private' stamp: 'dkh 10/3/2009 16:31'!
with: aMetacelloSpec during: aBlock 

	| previousRoot |
	previousRoot := self root.
	self root: aMetacelloSpec.
	aBlock
		ensure: [self root: previousRoot]! !

!Object methodsFor: '*metacello-core' stamp: 'dkh 8/13/2009 10:37'!
metacelloIntegerLessThanSelf: anInteger

	^self error: 'Invalid Metacello verson component - should be String or Integer.'! !

!Object methodsFor: '*metacello-core' stamp: 'dkh 8/13/2009 11:22'!
metacelloStringLessThanSelf: anInteger

	^self error: 'Invalid Metacello verson component - should be String or Integer.'! !

!Object methodsFor: '*metacello-core' stamp: 'dkh 8/13/2009 10:20'!
metacelloVersionComponentLessThan: aMetacelloVersonComponent

	^self error: 'Invalid Metacello verson component - should be String or Integer.'! !

Notification subclass: #MetacelloClearStackCacheNotification
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Exceptions'!

Notification subclass: #MetacelloStackCacheNotification
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Core-Exceptions'!

!DateAndTime methodsFor: '*metacello-core' stamp: 'dkh 9/13/2009 07:49'!
setTimestampInMetacelloVersion: aMetacelloVersionSpec

	aMetacelloVersionSpec setTimestamp: 
		(aMetacelloVersionSpec project valueHolderSpec
			value: self printString;
			yourself)! !
MetacelloPlatform initialize!
